@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.23 2000/03/16 15:56:07 abe Exp $
+ * $Id: machine.h,v 1.28 2000/12/05 20:25:13 abe Exp $
  */
 
 
@@ -69,6 +69,10 @@
 
 #include <sys/types.h>
 
+#   if	AIXV>=5199
+#include <sys/resource.h>
+#   endif	/* AIXV>=5199 */
+
 # if	defined(__GNUC__)
 typedef	long long	aligned_off64_t __attribute__ ((aligned (8)));
 typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
@@ -262,10 +266,16 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
  *
  * HASNCAPID is defined for those dialects with a searchable kernel name
  * cache whose cache and vnodes are linked by a capability ID.
+ *
+ * NCACHELDPFX is a set of C commands to execute before calling ncache_load().
+ *
+ * NCACHELDSFX is a set of C commands to execute after calling ncache_load().
  */
 
 /* #define	HASNCACHE	1	*/
 /* #define	HASNCAPID	1	*/
+/* #define	NCACHELDPFX	??? */
+/* #define	NCACHELDSFX	??? */
 
 
 /*
@@ -385,18 +395,25 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
  * by inode number.
  */
 
+#if	AIXV<5199
 /* #define	HASPROCFS	"proc?" */
 /* #define	HASFSTYPE	1 */
 /* #define	HASPINODEN	1 */
+#else	/* AIXV>=5199 */
+#define	HASPROCFS	"proc"
+#define	HASFSTYPE	2
+#define	HASPINODEN	1
+#endif	/* AIXV<5199 */
 
 
 /*
  * HASRNODE is defined for those dialects that have rnodes.
+ *
+ * Note: while AIX has rnodes, they are processed privately, so this
+ *	 definition should be disabled.
  */
 
-#if	defined(HAS_NFS)
-#define	HASRNODE	1
-#endif	/* defined(HAS_NFS) */
+/* #define	HASRNODE	1 */
 
 
 /*
@@ -416,6 +433,24 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
 /* #define	HASSNODE	1 */
 
 
+/*
+ * Define HASSPECDEVD to be the name of a function that handles the results
+ * of a successful stat(2) of a file name argument.
+ *
+ * For example, HASSPECDEVD() for Darwin makes sure that st_dev is set to
+ * what stat("/dev") returns -- i.e., what's in DevDev.
+ *
+ * The function takes two arguments:
+ *
+ *	1: pointer to the full path name of file
+ *	2: pointer to the stat(2) result
+ *
+ * The function returns void.
+ */
+
+/* #define	HASSPECDEVD	process_dev_stat */
+
+
 /*
  * HASSTREAMS is defined for those dialects that support streams.
  */
@@ -506,6 +541,8 @@ typedef	long long	aligned_offset_t  __attribute__ ((aligned (8)));
 /* #define	USE_LIB_RNAM			1	   rnam.c */
 /* #define	USE_LIB_RNCH			1	   rnch.c */
 /* #define	USE_LIB_RNMH			1	   rnmh.c */
+/* #define	USE_LIB_SNPF			1	   snpf.c */
+#define	snpf	snprintf	   /* use the system's snprintf() */
 
 
 /*
