@@ -1,5 +1,5 @@
 /*
- * dnode.c - Digital UNIX (DEC OSF/1) node functions for lsof
+ * dnode.c - DEC OSF/1, Digital UNIX, Tru64 UNIX node functions for lsof
  */
 
 
@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.5 98/02/16 19:48:11 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.16 2000/12/04 14:24:15 abe Exp $";
 #endif
 
 
@@ -43,6 +43,74 @@ static char *rcsid = "$Id: dnode.c,v 1.5 98/02/16 19:48:11 abe Exp $";
  * Local definitions
  */
 
+#if	ADVFSV>=500
+/*
+ * AdvFS (MSFS) definitions for AdvFS version 5.0 and above.
+ */
+
+struct fs_stat {			/* file system stat(2) info structure
+					 * for AdvFS 5.0 and above */
+	unsigned int num;		/* node number */
+	unsigned int d1;
+	mode_t d2;
+	uid_t d3;
+	gid_t d4;
+	dev_t rdev;			/* character or block device number */
+	off_t size;			/* file size */
+
+# if	defined(__arch32__)
+	unsigned int d5;
+# endif	/* defined(__arch32__) */
+
+	time_t d6;
+	int d7;
+	time_t d8;
+	int d9;
+	time_t d10;
+	int d11;
+	unsigned int d12[5];
+	unsigned short nlink;		/* link count */
+};
+
+struct fsContext {			/* file sysem context for AdvFS 5.0
+					 * and above */
+	short d1[2];
+	unsigned int d2[2];
+	long d3;
+	int d4[2];
+	lock_data_t d5;
+	long d6;
+	simple_lock_data_t d7;
+	unsigned int d8[2];
+	long d9;
+	struct fs_stat st;		/* file stats */
+};
+
+struct advfsnode {			/* AdvFS (MSFS) node definition for
+					 * AdvFS 5.0 and above */
+	unsigned long d1;
+	struct fsConext *a_con;		/* context pointer */
+};
+#endif	/* ADVFSV>=500 */
+
+#if	DUV>=50000
+typedef struct cnode {			/* CFS node structure definition for
+					 * Tru64 UNIX 5.0 and above */
+        udecl_simple_lock_data(, d1)
+	unsigned int d2;
+	time_t d3;
+	unsigned long d4[3];
+
+# if	DUV<50100
+	int d5[2];
+	off_t d6;
+# endif	/* DUV<50100 */
+
+	unsigned long d7;
+        vattr_t          c_attr;        /* 96:Cached vnode attributes */
+} cnode_t;
+#endif	/* DUV>=50000 */
+
 struct l_lock {				/* local lock info */
 	struct eflock set;		/* lock data */
 	struct l_lock *next;
@@ -74,7 +142,7 @@ static int FlinfoSt = 0;		/* Flinfo[] load status */
 
 _PROTOTYPE(static char isvlocked,(struct vnode *vp));
 _PROTOTYPE(static int load_flinfo,(void));
-_PROTOTYPE(static int readvnode,(caddr_t va, struct vnode *v));
+_PROTOTYPE(static int readvnode,(KA_T va, struct vnode *v));
 _PROTOTYPE(static void get_proc_sz,(struct procnode *pn));
 
 
@@ -104,6 +172,7 @@ clr_flinfo()
 		fin = fi->next;
 		(void) free((FREE_P *)fi);
 	    } while ((fi = fin));
+	    Flinfo[i] = (struct l_flinfo *)NULL;
 	}
 	FlinfoSt = 0;
 }
@@ -216,7 +285,7 @@ load_flinfo()
 	int i;
 	struct l_flinfo *lfi;
 	struct l_lock *ll;
-	unsigned long v;
+	KA_T v;
 
 	if (Flinfo && FlinfoSt)
 	    return(1);
@@ -302,12 +371,13 @@ load_flinfo()
 
 void
 process_node(va)
-	caddr_t va;			/* vnode kernel space address */
+	KA_T va;			/* vnode kernel space address */
 {
 	struct advfsnode *a = (struct advfsnode *)NULL;
 	struct cdnode *c = (struct cdnode *)NULL;
-	dev_t dev;
-	int devs;
+	dev_t dev, rdev;
+	unsigned char devs = 0;
+	unsigned char rdevs = 0;
 	struct inode *i = (struct inode *)NULL;
 	struct mfsnode *m = (struct mfsnode *)NULL;
 	struct procnode *p = (struct procnode *)NULL;
@@ -318,195 +388,302 @@ process_node(va)
 	struct s5inode *s5 = (struct s5inode *)NULL;
 	char *ty;
 	enum vtype type;
+	unsigned long ul;
 	static struct vnode *v = (struct vnode *)NULL;
 	struct l_vfs *vfs;
 
 #if	DUV>=30000
-	char dev_ch[32];
 	struct fifonode *f = (struct fifonode *)NULL;
 	struct fifonode fn;
+	static struct vnode *fv = (struct vnode *)NULL;
 #endif	/* DUV>=30000 */
 
+#if	DUV>=50000
+	cnode_t *cn = (cnode_t *)NULL;
+	struct fsContext fsc;
+	int fscs = 0;
+#endif	/* DUV>=50000 */
+
 /*
  * Read the vnode.
  */
-	if ( ! va) {
-		enter_nm("no vnode address");
-		return;
+	if (!va) {
+	    enter_nm("no vnode address");
+	    return;
 	}
 	if (!v) {
 
 	/*
 	 * Allocate space for the Digital UNIX vnode.
 	 */
-		if (!(v = (struct vnode *)malloc(sizeof(struct vnode)-1+Vnmxp)))
-		{
-			(void) fprintf(stderr,
-				"%s: no space for vnode buffer\n", Pn);
-			Exit(1);
-		}
+	    if (!(v = (struct vnode *)malloc(sizeof(struct vnode)-1+Vnmxp))) {
+		(void) fprintf(stderr, "%s: no space for vnode buffer\n", Pn);
+		Exit(1);
+	    }
+
+#if	DUV>=30000
+	    if (!(fv = (struct vnode *)malloc(sizeof(struct vnode)-1+Vnmxp))) {
+		(void) fprintf(stderr, "%s: no space for fvnode buffer\n", Pn);
+		Exit(1);
+	    }
+#endif	/* DUV>=30000 */
+
 	}
 	if (readvnode(va, v)) {
-                enter_nm(Namech);
-                return;
-        }
+	    enter_nm(Namech);
+	    return;
+	}
 
-# if	defined(HASNCACHE)
+#if	defined(HASNCACHE)
+	Lf->na = va;
+# if	defined(HASNCAPID)
 	Lf->id = (unsigned long)v->v_id;
-	Lf->na = (unsigned long)va;
-# endif	/* defined(HASNCACHE) */
+# endif	/* defined(HASNCAPID) */
+#endif	/* defined(HASNCACHE) */
+
+#if	defined(HASFSTRUCT)
+	Lf->fsv |= FSV_NI;
+	Lf->fna = va;
+#endif	/* defined(HASFSTRUCT) */
 
 /*
  * Get the mount structure and determine the vnode type.
  */
 	if (!v->v_mount)
-		vfs = (struct l_vfs *)NULL;
+	    vfs = (struct l_vfs *)NULL;
 	else
-		vfs = readvfs(v->v_mount);
+	    vfs = readvfs((KA_T)v->v_mount);
 	if (vfs) {
-		switch (vfs->type) {
-		case MOUNT_NFS:
+	    switch (vfs->type) {
+	    case MOUNT_NFS:
 
 #if	defined(MOUNT_NFS3)
-		case MOUNT_NFS3:
+	    case MOUNT_NFS3:
 #endif	/* defined(MOUNT_NFS3) */
 
-			Ntype = N_NFS;
-			break;
-		}
-		if (Ntype == N_REGLR) {
-			switch (v->v_type) {
-			case VFIFO:
-				Ntype = N_FIFO;
-				break;
-			}
+		Ntype = N_NFS;
+		break;
+	    }
+	    if (Ntype == N_REGLR) {
+		switch (v->v_type) {
+		case VFIFO:
+		    Ntype = N_FIFO;
+		    break;
 		}
+	    }
 	}
 /*
  * Determine the lock type.
  */
 	if (FILEPTR && (FILEPTR->f_flag & FSHLOCK))
-		Lf->lock = 'R';
+	    Lf->lock = 'R';
 	else if (FILEPTR && (FILEPTR->f_flag & FEXLOCK))
-		Lf->lock = 'W';
+	    Lf->lock = 'W';
 	else
-		Lf->lock = isvlocked((struct vnode *)va);
+	    Lf->lock = isvlocked((struct vnode *)va);
 /*
  * Define the specific Digital UNIX node pointer.
  */
 
 #if	DUV>=30000
 	if (Ntype == N_FIFO) {
-		if (v->v_fifonode
-		&&  kread((KA_T)v->v_fifonode, (char *)&fn, sizeof(fn)) == 0)
-			f = &fn;
+	    if (v->v_fifonode
+	    &&  !kread((KA_T)v->v_fifonode, (char *)&fn, sizeof(fn)))
+		f = &fn;
 	}
 #endif	/* DUV>=30000 */
 
 	switch (v->v_tag) {
 	case VT_CDFS:
-		c = (struct cdnode *)v->v_data;
-		break;
+	    c = (struct cdnode *)v->v_data;
+	    break;
+
+#if	DUV>=50000
+	case VT_CFS:
+	    cn = (cnode_t *)v->v_data;
+	    break;
+#endif	/* DUV>=50000 */
+
 	case VT_MFS:
-		m = (struct mfsnode *)v->v_data;
-		break;
+	    m = (struct mfsnode *)v->v_data;
+	    break;
 	case VT_NFS:
-		r = (struct rnode *)v->v_data;
-		break;
+	    r = (struct rnode *)v->v_data;
+	    break;
 	case VT_NON:
 
 #if     DUV<20000
-		if (v->v_specinfo
-		&&  kread((KA_T)v->v_specinfo, (char *)&sn, sizeof(sn)) == 0)
-			s = &sn;
-		else
+	    if (v->v_specinfo
+	    &&  !kread((KA_T)v->v_specinfo, (char *)&sn, sizeof(sn)))
+		s = &sn;
+	    else
+#else	/* DUV>=20000 */
+# if	DUV>=30000
+	    if (!f)
+# endif	/* DUV>=30000 */
 #endif  /* DUV<20000 */
 
-			s = (struct spec_node *)v->v_data;
-		break;
+		s = (struct spec_node *)v->v_data;
+	    break;
 	case VT_PRFS:
-		p = (struct procnode *)v->v_data;
-		break;
+	    p = (struct procnode *)v->v_data;
+	    break;
 	case VT_S5FS:
-		s5 = (struct s5inode *)v->v_data;
-		break;
+	    s5 = (struct s5inode *)v->v_data;
+	    break;
 	case VT_MSFS:
-		a = (struct advfsnode *)v->v_data;
-		break;
+	    a = (struct advfsnode *)v->v_data;
+
+#if	ADVFSV>=500
+	    if (a->a_con
+	    &&  !kread((KA_T)a->a_con, (char *)&fsc, sizeof(fsc)))
+		fscs = 1;
+#endif	/* ADVFSV>=500 */
+
+	    break;
 	case VT_UFS:
-		i = (struct inode *)v->v_data;
-		break;
+	    i = (struct inode *)v->v_data;
+	    break;
+	default:
+	    (void) snpf(Namech, Namechl, "unknown node type, v_tag=%d",
+		v->v_tag);
+	    enter_nm(Namech);
+	    return;
 	}
 /*
  * Get device and type for printing.
  */
 	type = v->v_type;
-	dev = devs = 0;
 	if (a) {
-		if (type == VCHR) {
-			dev = a->a_rdev;
-			devs = 1;
-		} else if (vfs) {
-			dev = vfs->dev;
-			devs = 1;
+	    if (vfs && vfs->dev) {
+		dev = vfs->dev;
+		devs = 1;
+	    }
+	    if ((type == VCHR) || (type == VBLK)) {
+
+#if	ADVFSV>=500
+		if (fscs) {
+		    rdev = fsc.st.rdev;
+		    rdevs = 1;
 		}
+#else	/* ADVFSV<500 */
+		rdev = a->a_rdev;
+		rdevs = 1;
+#endif	/* ADVFSV>=500 */
+
+	    }
 	} else if (c) {
-		dev = c->cd_dev;
+	    dev = c->cd_dev;
+	    devs = 1;
+	}
+
+#if	DUV>=50000
+	else if (cn) {
+	    if (vfs && vfs->dev) {
+		dev = vfs->dev;
 		devs = 1;
-	} else if (i) {
-		dev = (type == VCHR) ? i->i_din.di_db[0] : i->i_dev;
+	    }
+	    if ((type == VCHR) || (type == VBLK)) {
+		if (cn->c_attr.va_mask & AT_RDEV) {
+		    rdev = cn->c_attr.va_rdev;
+		    rdevs = 1;
+		}
+	    }
+	}
+#endif	/* DUV>=50000 */
+
+	else if (i) {
+	    if (i->i_dev) {
+		dev = i->i_dev;
 		devs = 1;
+	    } else if (vfs && vfs->dev) {
+		dev = vfs->dev;
+		devs = 1;
+	    }
+	    if ((type == VCHR) || (type == VBLK)) {
+		rdev = i->i_din.di_db[0];
+		rdevs = 1;
+	    }
 	} else if (r) {
-		dev = r->r_attr.va_fsid;
-		devs = 1;
+	    dev = r->r_attr.va_fsid;
+	    devs = 1;
 	} else if (s) {
-		devs = 1;
-		dev = s->sn_vattr.va_rdev;
-		if ( ! lkupdev(&dev, 0) && Clonedev != -1)
-			dev = makedev(major(Clonedev), major(dev));
+	    if (vfs && vfs->dev)
+		dev = vfs->dev;
+	    else
+		dev = DevDev;
+	    devs = 1;
+	    rdev = s->sn_vattr.va_rdev;
+	    rdevs = 1;
+	    if (!lkupdev(&dev, &rdev, 0, 0) && HaveCloneMaj)
+		rdev = makedev(CloneMaj, major(rdev));
 	} else if (s5) {
-		dev = s5->i_dev;
+	    dev = s5->i_dev;
+	    devs = 1;
+	} else if (f) {
+	    if (vfs && vfs->dev) {
+		dev = vfs->dev;
 		devs = 1;
+	    }
 	}
 /*
  * Obtain the inode number.
  */
 	if (a) {
-		Lf->inode = (unsigned long)a->a_number;
-		Lf->inp_ty = 1;
 
-#if	DUV>=40000 && ADVFSV>=400
+#if	ADVFSV>=500
+	    if (fscs) {
+		Lf->inode = (unsigned long)fsc.st.num;
+		Lf->inp_ty = 1;
+	    }
+#else	/* ADVFSV<500 */
+	    Lf->inode = (unsigned long)a->a_number;
+	    Lf->inp_ty = 1;
+#endif	/* ADVFSV>=500 */
+
+
+#if	defined(HASTAGTOPATH)
 	/*
 	 * Record the Digital UNIX 4.0 or greater, ADVFS 4.0 or greater
 	 * ADVFS sequence number for later use with tag_to_path().
 	 */
-		Lf->advfs_seq = a->a_seq;
-		Lf->advfs_seq_stat = 1;
-#endif	/* DUV>=40000 && ADVFSV>=400 */
+	    Lf->advfs_seq = a->a_seq;
+	    Lf->advfs_seq_stat = 1;
+#endif	/* defined(HASTAGTOPATH) */
 
 	} else if (c) {
-		Lf->inode = (unsigned long)c->cd_number;
-		Lf->inp_ty = 1;
+	    Lf->inode = (unsigned long)c->cd_number;
+	    Lf->inp_ty = 1;
 	}
+
+#if	DUV>=50000
+	else if (cn) {
+	    if (cn->c_attr.va_mask & AT_NODEID) {
+		Lf->inode = (unsigned long)cn->c_attr.va_fileid;
+		Lf->inp_ty = 1;
+	    }
+	}
+#endif	/* DUV>=50000 */
+
 	else if (i) {
-		Lf->inode = (unsigned long)i->i_number;
-		Lf->inp_ty = 1;
+	    Lf->inode = (unsigned long)i->i_number;
+	    Lf->inp_ty = 1;
 	} else if (p) {
-		Lf->inode = (unsigned long)((type == VDIR) ? PR_ROOTINO
-					   : p->prc_pid + PR_INOBIAS);
-		Lf->inp_ty = 1;
+	    Lf->inode = (unsigned long)((type == VDIR) ? PR_ROOTINO
+					: p->prc_pid + PR_INOBIAS);
+	    Lf->inp_ty = 1;
 	} else if (r) {
-		Lf->inode = (unsigned long)r->r_attr.va_fileid;
-		Lf->inp_ty = 1;
+	    Lf->inode = (unsigned long)r->r_attr.va_fileid;
+	    Lf->inp_ty = 1;
 	} else if (s5) {
-		Lf->inode = (unsigned long)s5->i_number;
-		Lf->inp_ty = 1;
+	    Lf->inode = (unsigned long)s5->i_number;
+	    Lf->inp_ty = 1;
 	}
 
 #if	DUV>=30000
 	else if (f) {
-		Lf->inode = (unsigned long)f->fn_fileid;
-		Lf->inp_ty = 1;
+	    Lf->inode = (unsigned long)f->fn_fileid;
+	    Lf->inp_ty = 1;
 	}
 #endif	/* DUV>=30000 */
 
@@ -514,142 +691,211 @@ process_node(va)
  * Obtain the file size.
  */
 	if (Foffset) {
-		Lf->off_def = 1;
+	    Lf->off_def = 1;
 
 #if	DUV>=30000
-		if (Ntype == N_FIFO && f)
-			Lf->off = (unsigned long)
-				  (Lf->access == 'r') ? f->fn_rptr
-						      : f->fn_wptr;
+	    if (Ntype == N_FIFO && f)
+		Lf->off = (unsigned long)
+		    (Lf->access == 'r') ? f->fn_rptr : f->fn_wptr;
 #endif	/* DUV>=30000 */
 
 	} else {
-		switch (Ntype) {
-		case N_FIFO:
+	    switch (Ntype) {
+	    case N_FIFO:
 
 #if	DUV>=30000
-			if (f) {
-				Lf->sz = (SZOFFTYPE)f->fn_size;
-				Lf->sz_def = 1;
-			} else if (!Fsize)
-				Lf->off_def = 1;
+		if (f) {
+		    Lf->sz = (SZOFFTYPE)f->fn_size;
+		    Lf->sz_def = 1;
+		} else if (!Fsize)
+		    Lf->off_def = 1;
 #else	/* DUV<30000 */
-			if (!Fsize)
-				Lf->off_def = 1;
+		if (!Fsize)
+		    Lf->off_def = 1;
 #endif	/* DUV>=30000 */
 
-			break;
-		case N_NFS:
-			if (r) {
-				Lf->sz = (SZOFFTYPE)r->r_attr.va_qsize;
-				Lf->sz_def = 1;
-			}
-			break;
-		case N_REGLR:
-			if (type == VREG || type == VDIR) {
-				if (a) {
-					Lf->sz = (SZOFFTYPE)a->a_size;
-					Lf->sz_def = 1;
-				} else if (c) {
-					Lf->sz = (SZOFFTYPE)c->cd_size;
-					Lf->sz_def = 1;
-				} else if (i) {
-					Lf->sz = (SZOFFTYPE)i->i_din.di_qsize;
-					Lf->sz_def = 1;
-				} else if (m) {
-					Lf->sz = (SZOFFTYPE)m->mfs_size;
-					Lf->sz_def = 1;
-				} else if (p) {
-					if (type != VDIR)
-						get_proc_sz(p);
-				} else if (s5) {
-					Lf->sz = (SZOFFTYPE)s5->i_size;
-					Lf->sz_def = 1;
-				}
-			} else if (type == VCHR && !Fsize)
-				Lf->off_def = 1;
+		break;
+	    case N_NFS:
+		if (r) {
+		    Lf->sz = (SZOFFTYPE)r->r_attr.va_qsize;
+		    Lf->sz_def = 1;
 		}
+		break;
+	    case N_REGLR:
+		if (type == VREG || type == VDIR) {
+		    if (a) {
+
+#if	ADVFSV>=500
+			if (fscs) {
+			    Lf->sz = (SZOFFTYPE)fsc.st.size;
+			    Lf->sz_def = 1;
+			}
+#else	/* ADVFSV<500 */
+			Lf->sz = (SZOFFTYPE)a->a_size;
+			Lf->sz_def = 1;
+#endif	/* ADVFSV>=500 */
+
+		    } else if (c) {
+			Lf->sz = (SZOFFTYPE)c->cd_size;
+			Lf->sz_def = 1;
+		    }
+
+#if	DUV>=50000
+		    else if (cn) {
+			if (cn->c_attr.va_mask & AT_SIZE) {
+			    Lf->sz = (SZOFFTYPE)cn->c_attr.va_qsize;
+			    Lf->sz_def = 1;
+			}
+		    }
+#endif	/* DUV>=50000 */
+
+		    else if (i) {
+			Lf->sz = (SZOFFTYPE)i->i_din.di_qsize;
+			Lf->sz_def = 1;
+		    } else if (m) {
+			Lf->sz = (SZOFFTYPE)m->mfs_size;
+			Lf->sz_def = 1;
+		    } else if (p) {
+			if (type != VDIR)
+				get_proc_sz(p);
+		    } else if (s5) {
+			Lf->sz = (SZOFFTYPE)s5->i_size;
+			Lf->sz_def = 1;
+		    }
+		} else if (type == VCHR && !Fsize)
+		    Lf->off_def = 1;
+	    }
+	}
+	if (Fnlink) {
+	    switch(Ntype) {
+	    case N_FIFO:			/* no link count */
+		break;
+	    case N_NFS:
+		Lf->nlink = (long)r->r_attr.va_nlink;
+		Lf->nlink_def = 1;
+		break;
+	    case N_REGLR:
+
+#  if	ADVFSV>=400
+		if (a) {
+
+#if	ADVFSV>=500
+		    if (fscs) {
+			Lf->nlink = (long)fsc.st.nlink;
+			Lf->nlink_def = 1;
+		    }
+#else	/* ADVFSV<500 */
+		    Lf->nlink = (long)a->a_nlink;
+		    Lf->nlink_def = 1;
+#endif	/* ADVFSV>=500 */
+
+		    break;
+		}
+#  endif	/* ADVFSV>=400 */
+
+		if (c) {
+		    Lf->nlink = (long)c->cd_nlink;
+		    Lf->nlink_def = 1;
+		}
+
+#if	DUV>=50000
+		else if (cn) {
+		    if (cn->c_attr.va_mask & AT_NLINK) {
+			Lf->nlink = (long)cn->c_attr.va_nlink;
+			Lf->nlink_def = 1;
+		    }
+		}
+#endif	/* DUV>=50000 */
+
+		else if (i) {
+		    Lf->nlink = (long)i->i_din.di_nlink;
+		    Lf->nlink_def = 1;
+		} else if (s5) {
+		    Lf->nlink = (long)s5->i_nlink;
+		    Lf->nlink_def = 1;
+		}
+	    }
+	    if (Nlink && Lf->nlink_def && (Lf->nlink < Nlink))
+		Lf->sf |= SELNLINK;
 	}
 /*
  * Record an NFS file selection.
  */
 	if (Ntype == N_NFS && Fnfs)
-		Lf->sf |= SELNFS;
+	    Lf->sf |= SELNFS;
 /*
+ * Save the device numbers and their states.
+ *
  * Format the vnode type, and possibly the device name.
  */
+	Lf->dev = dev;
+	Lf->dev_def = devs;
+	Lf->rdev = rdev;
+	Lf->rdev_def = rdevs;
 	switch (type) {
-
 	case VNON:
-		ty ="VNON";
-		break;
+	    ty ="VNON";
+	    break;
 	case VREG:
 	case VDIR:
-		ty = (type == VREG) ? "VREG" : "VDIR";
-		Lf->dev_def = 1;
-		Lf->dev = dev;
-		break;
+	    ty = (type == VREG) ? "VREG" : "VDIR";
+	    break;
 	case VBLK:
-		ty = "VBLK";
-		break;
+	    ty = "VBLK";
+	    break;
 	case VCHR:
-		ty = "VCHR";
-		Lf->dev = dev;
-		Lf->dev_def = 1;
-		Ntype = N_CHR;
-		break;
+	    ty = "VCHR";
+	    Ntype = N_CHR;
+	    break;
 	case VLNK:
-		ty = "VLNK";
-		break;
+	    ty = "VLNK";
+	    break;
 
 #if	defined(VSOCK)
 	case VSOCK:
-		ty = "SOCK";
-		break;
-#endif
+	    ty = "SOCK";
+	    break;
+#endif	/* defined(VSOCK) */
 
 	case VBAD:
-		ty = "VBAD";
-		break;
+	    ty = "VBAD";
+	    break;
 	case VFIFO:
-		if (devs && dev) {
-			Lf->dev = dev;
-			Lf->dev_def = 1;
-		}
+	    ty = "FIFO";
 
 #if	DUV>=30000
-		else if (f) {
-			vfs = (struct l_vfs *)NULL;
-			(void) sprintf(dev_ch, "0x%08x", v->v_fifonode);
-			(void) enter_dev_ch(dev_ch);
-		}
+	    if ((!devs || !dev) && f) {
+		vfs = (struct l_vfs *)NULL;
+		devs = Lf->dev_def = 0;
+		ul = (unsigned long)v->v_fifonode;
+		enter_dev_ch(print_kptr((KA_T)(ul&0xffffffff),(char *)NULL,0));
+	    }
 #endif	/* DUV>=30000 */
 
-		ty = "FIFO";
-		break;
+	    break;
 	default:
-		if (type > 9999)
-			(void) sprintf(Lf->type, "*%03d", type % 1000);
-		else
-			(void) sprintf(Lf->type, "%4d", type);
-		(void) strcpy(Namech, "unknown type");
-		ty = (char *)NULL;
+	    if (type > 9999)
+		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
+	    else
+		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
+	    (void) snpf(Namech, Namechl, "unknown type");
+	    ty = (char *)NULL;
 	}
 	if (ty)
-		(void) strcpy(Lf->type, ty);
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%s", ty);
 	Lf->ntype = Ntype;
 /*
  * Save the file system names.
  */
 	if (vfs) {
-		if (vfs->dir && *vfs->dir)
-			Lf->fsdir = vfs->dir;
-		if (vfs->fsname && *vfs->fsname)
-			Lf->fsdev = vfs->fsname;
+	    if (vfs->dir && *vfs->dir)
+		Lf->fsdir = vfs->dir;
+	    if (vfs->fsname && *vfs->fsname)
+		Lf->fsdev = vfs->fsname;
 
 #if	defined(HASFSINO)
-		if (vfs->fs_ino)
-			Lf->fs_ino = vfs->fs_ino;
+	    if (vfs->fs_ino)
+		Lf->fs_ino = vfs->fs_ino;
 #endif	/* defined(HASFSINO) */
 
 	}
@@ -663,24 +909,34 @@ process_node(va)
  */
 
 	if (type == VBAD)
-		(void) strcpy(Namech, "(revoked)");
+	    (void) snpf(Namech, Namechl, "(revoked)");
 	if (m) {
-		Lf->dev_def = 0;
-		(void) sprintf(Namech, "%#x", m->mfs_baseoff);
-		(void) enter_dev_ch("    memory");
+	    devs = Lf->dev_def = Lf->rdev_def = rdevs = 0;
+	    (void) snpf(Namech, Namechl, "%#x", m->mfs_baseoff);
+	    (void) enter_dev_ch("memory");
 	} else if (p) {
-		Lf->dev_def = 0;
-		if (type != VDIR)
-			(void) sprintf(Namech, "/proc/%d", p->prc_pid);
-		else
-			(void) strcpy(Namech, "/proc");
+	    devs = Lf->dev_def = Lf->rdev_def = rdevs = 0;
+	    if (type != VDIR)
+		(void) snpf(Namech, Namechl, "/proc/%d", p->prc_pid);
+	    else
+		(void) snpf(Namech, Namechl, "/proc");
 	}
+
+#if	defined(HASBLKDEV)
+/*
+ * If this is a VBLK file and it's missing an inode number, try to
+ * supply one.
+ */
+	if ((Lf->inp_ty == 0) && (type == VBLK))
+	    find_bl_ino();
+#endif	/* defined(HASBLKDEV) */
+
 /*
  * If this is a VCHR file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && type == VCHR && Lf->dev_def)
-		find_ch_ino();
+	if ((Lf->inp_ty == 0) && (type == VCHR))
+	    find_ch_ino();
 /*
  * Test for specified file.
  */
@@ -704,14 +960,14 @@ process_node(va)
 		}
 	    }
 	} else {
-		if (Sfile && is_file_named((char *)NULL, type))
-			Lf->sf |= SELNM;
+	    if (Sfile && is_file_named((char *)NULL, (type == VCHR) ? 1 : 0))
+		Lf->sf |= SELNM;
 	}
 /*
  * Enter name characters.
  */
 	if (Namech[0])
-		enter_nm(Namech);
+	    enter_nm(Namech);
 }
 
 
@@ -721,13 +977,14 @@ process_node(va)
 
 static int
 readvnode(va, v)
-	caddr_t va;			/* vnode kernel space address */
+	KA_T va;			/* vnode kernel space address */
 	struct vnode *v;		/* vnode buffer pointer */
 {
 
 	if (kread((KA_T)va, (char *)v, sizeof(struct vnode) - 1 + Vnmxp)) {
-		(void) sprintf(Namech, "can't read vnode at %#x", va);
-		return(1);
+	    (void) snpf(Namech, Namechl, "can't read vnode at %s",
+		print_kptr(va, (char *)NULL, 0));
+	    return(1);
 	}
 	return(0);
 }
