@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.12 2000/05/31 13:03:00 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.14 2000/07/31 19:13:29 abe Exp $";
 #endif
 
 
@@ -78,7 +78,7 @@ process_socket(sa)
 	struct in6pcb in6p;
 #endif	/* defined(HASIPv6) && !defined(HASINRIAIPv6) */
 
-	(void) strcpy(Lf->type, "sock");
+	(void) snpf(Lf->type, sizeof(Lf->type), "sock");
 	Lf->inp_ty = 2;
 /*
  * Read the socket, protocol, and domain structures.
@@ -88,8 +88,8 @@ process_socket(sa)
 	    return;
 	}
 	if (kread(sa, (char *) &s, sizeof(s))) {
-	    (void) sprintf(Namech, "can't read socket struct from %s",
-		print_kptr(sa, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read socket struct from %s",
+		print_kptr(sa, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
@@ -99,15 +99,15 @@ process_socket(sa)
 	}
 	if (!s.so_proto
 	||  kread((KA_T)s.so_proto, (char *)&p, sizeof(p))) {
-	    (void) sprintf(Namech, "can't read protocol switch from %s",
-		print_kptr((KA_T)s.so_proto, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read protocol switch from %s",
+		print_kptr((KA_T)s.so_proto, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
 	if (!p.pr_domain
 	||  kread((KA_T)p.pr_domain, (char *)&d, sizeof(d))) {
-	    (void) sprintf(Namech, "can't read domain struct from %s",
-		print_kptr((KA_T)p.pr_domain, (char *)NULL));
+	    (void) snpf(Namech, Namechl, "can't read domain struct from %s",
+		print_kptr((KA_T)p.pr_domain, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return;
 	}
@@ -149,9 +149,10 @@ process_socket(sa)
 	    printiproto(p.pr_protocol);
 
 #if	defined(HASIPv6)
-	    (void) strcpy(Lf->type, (fam == AF_INET) ? "IPv4" : "IPv6");
+	    (void) snpf(Lf->type, sizeof(Lf->type),
+		(fam == AF_INET) ? "IPv4" : "IPv6");
 #else	/* !defined(HASIPv6) */
-	    (void) strcpy(Lf->type, "inet");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "inet");
 #endif	/* defined(HASIPv6) */
 
 #if	defined(HASIPv6) && !defined(HASINRIAIPv6)
@@ -162,8 +163,8 @@ process_socket(sa)
 	     */
 		if (!s.so_pcb
 		||  kread((KA_T)s.so_pcb, (char *)&in6p, sizeof(in6p))) {
-		    (void) sprintf(Namech, "can't read in6pcb at %s",
-			print_kptr((KA_T)s.so_pcb, (char *)NULL));
+		    (void) snpf(Namech, Namechl, "can't read in6pcb at %s",
+			print_kptr((KA_T)s.so_pcb, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
@@ -172,7 +173,7 @@ process_socket(sa)
 	     */
 		enter_dev_ch(print_kptr((KA_T)(in6p.in6p_ppcb ? in6p.in6p_ppcb
 							      : s.so_pcb),
-					       (char *)NULL));
+					       (char *)NULL, 0));
 		if (p.pr_protocol == IPPROTO_TCP)
 		    ta = (KA_T)in6p.in6p_ppcb;
 	        la = (unsigned char *)&in6p.in6p_laddr;
@@ -194,21 +195,21 @@ process_socket(sa)
 		if (!s.so_pcb
 		||  kread((KA_T) s.so_pcb, (char *) &inp, sizeof(inp))) {
 		    if (!s.so_pcb) {
-			(void) strcpy(Namech, "no PCB");
-			if (s.so_state & SS_CANTSENDMORE)
-			    (void) strcpy(endnm(), ", CANTSENDMORE");
-			if (s.so_state & SS_CANTRCVMORE)
-			    (void) strcpy(endnm(), ", CANTRCVMORE");
+			(void) snpf(Namech, Namechl, "no PCB%s%s",
+			    (s.so_state & SS_CANTSENDMORE) ? ", CANTSENDMORE"
+							   : "",
+			    (s.so_state & SS_CANTRCVMORE) ? ", CANTRCVMORE"
+							  : "");
 		    } else {
-			(void) sprintf(Namech, "can't read inpcb at %s",
-			    print_kptr((KA_T)s.so_pcb, (char *)NULL));
+			(void) snpf(Namech, Namechl, "can't read inpcb at %s",
+			    print_kptr((KA_T)s.so_pcb, (char *)NULL, 0));
 		    }
 		    enter_nm(Namech);
 		    return;
 		}
 		enter_dev_ch(print_kptr((KA_T)(inp.inp_ppcb ? inp.inp_ppcb
 							    : s.so_pcb),
-					       (char *)NULL));
+					       (char *)NULL, 0));
 		if (p.pr_protocol == IPPROTO_TCP)
 		    ta = (KA_T)inp.inp_ppcb;
 		lp = (int)ntohs(inp.inp_lport);
@@ -272,11 +273,11 @@ process_socket(sa)
  * Process a ROUTE domain socket.
  */
 	case AF_ROUTE:
-	    (void) strcpy(Lf->type, "rte");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "rte");
 	    if (s.so_pcb)
-		enter_dev_ch(print_kptr((KA_T)(s.so_pcb), (char *)NULL));
+		enter_dev_ch(print_kptr((KA_T)(s.so_pcb), (char *)NULL, 0));
 	    else
-		(void) strcpy(Namech, "no protocol control block");
+		(void) snpf(Namech, Namechl, "no protocol control block");
 	    if (!Fsize)
 		Lf->off_def = 1;
 	    break;
@@ -286,26 +287,26 @@ process_socket(sa)
 	case AF_UNIX:
 	    if (Funix)
 		Lf->sf |= SELUNX;
-	    (void) strcpy(Lf->type, "unix");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "unix");
 	/*
 	 * Read Unix protocol control block and the Unix address structure.
 	 */
 
-	    enter_dev_ch(print_kptr(sa, (char *)NULL));
+	    enter_dev_ch(print_kptr(sa, (char *)NULL, 0));
 	    if (kread((KA_T) s.so_pcb, (char *) &unp, sizeof(unp))) {
-		(void) sprintf(Namech, "can't read unpcb at %s",
-		    print_kptr((KA_T)s.so_pcb, (char *)NULL));
+		(void) snpf(Namech, Namechl, "can't read unpcb at %s",
+		    print_kptr((KA_T)s.so_pcb, (char *)NULL, 0));
 		break;
 	    }
 	    if ((struct socket *)sa != unp.unp_socket) {
-		(void) sprintf(Namech, "unp_socket (%s) mismatch",
-		    print_kptr((KA_T)unp.unp_socket, (char *)NULL));
+		(void) snpf(Namech, Namechl, "unp_socket (%s) mismatch",
+		    print_kptr((KA_T)unp.unp_socket, (char *)NULL, 0));
 		break;
 	    }
 	    if (unp.unp_addr) {
 		if (kread((KA_T) unp.unp_addr, (char *) &mb, sizeof(mb))) {
-		    (void) sprintf(Namech, "can't read unp_addr at %s",
-			print_kptr((KA_T)unp.unp_addr, (char *)NULL));
+		    (void) snpf(Namech, Namechl, "can't read unp_addr at %s",
+			print_kptr((KA_T)unp.unp_addr, (char *)NULL, 0));
 		    break;
 		}
 		if (mb.m_hdr.mh_len == sizeof(struct sockaddr_un))
@@ -326,15 +327,16 @@ process_socket(sa)
 		if (ua->sun_family == AF_UNSPEC) {
 		    if (unp.unp_conn) {
 			if (kread((KA_T)unp.unp_conn, (char *)&uc, sizeof(uc)))
-			    (void) sprintf(Namech, "can't read unp_conn at %s",
-				print_kptr((KA_T)unp.unp_conn, (char *)NULL));
+			    (void) snpf(Namech, Namechl,
+				"can't read unp_conn at %s",
+				print_kptr((KA_T)unp.unp_conn,(char *)NULL,0));
 			else
-			    (void) sprintf(Namech, "->%s",
-				print_kptr((KA_T)uc.unp_socket, (char *)NULL));
+			    (void) snpf(Namech, Namechl, "->%s",
+				print_kptr((KA_T)uc.unp_socket,(char *)NULL,0));
 		    } else
-			(void) strcpy(Namech, "->(none)");
+			(void) snpf(Namech, Namechl, "->(none)");
 		} else
-		    (void) sprintf(Namech, "unknown sun_family (%d)",
+		    (void) snpf(Namech, Namechl, "unknown sun_family (%d)",
 			ua->sun_family);
 		break;
 	    }
@@ -345,9 +347,9 @@ process_socket(sa)
 		if (Sfile && is_file_named(ua->sun_path, 0))
 		    Lf->sf |= SELNM;
 		if (!Namech[0])
-		    (void) strcpy(Namech, ua->sun_path);
+		    (void) snpf(Namech, Namechl, "%s", ua->sun_path);
 	    } else
-		(void) strcpy(Namech, "no address");
+		(void) snpf(Namech, Namechl, "no address");
 	    break;
 	default:
 	    printunkaf(fam, 1);
