@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.21 2000/03/16 15:56:19 abe Exp $
+ * $Id: machine.h,v 1.25 2000/12/05 20:27:25 abe Exp $
  */
 
 
@@ -216,10 +216,16 @@
  *
  * HASNCAPID is defined for those dialects with a searchable kernel name
  * cache whose cache and vnodes are linked by a capability ID.
+ *
+ * NCACHELDPFX is a set of C commands to execute before calling ncache_load().
+ *
+ * NCACHELDSFX is a set of C commands to execute after calling ncache_load().
  */
 
 #define	HASNCACHE	1
 #define	HASNCAPID	1
+/* #define	NCACHELDPFX	??? */
+/* #define	NCACHELDSFX	??? */
 
 
 /*
@@ -377,6 +383,24 @@
 /* #define	HASSNODE	1 */
 
 
+/*
+ * Define HASSPECDEVD to be the name of a function that handles the results
+ * of a successful stat(2) of a file name argument.
+ *
+ * For example, HASSPECDEVD() for Darwin makes sure that st_dev is set to
+ * what stat("/dev") returns -- i.e., what's in DevDev.
+ *
+ * The function takes two arguments:
+ *
+ *	1: pointer to the full path name of file
+ *	2: pointer to the stat(2) result
+ *
+ * The function returns void.
+ */
+
+/* #define	HASSPECDEVD	process_dev_stat */
+
+
 /*
  * HASSTREAMS is defined for those dialects that support streams.
  */
@@ -462,6 +486,8 @@
 # endif	/* (defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1020) */
 
 /* #define	USE_LIB_RNCH			1	   rnch.c */
+/* #define	USE_LIB_SNPF			1	   snpf.c */
+#define	snpf	snprintf	   /* use the system's snprintf() */
 
 
 /*
