@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: machine.h,v 1.20 2000/03/16 15:56:24 abe Exp $
+ * $Id: machine.h,v 1.23 2000/12/05 20:23:51 abe Exp $
  */
 
 
@@ -222,10 +222,16 @@
  *
  * HASNCAPID is defined for those dialects with a searchable kernel name
  * cache whose cache and vnodes are linked by a capability ID.
+ *
+ * NCACHELDPFX is a set of C commands to execute before calling ncache_load().
+ *
+ * NCACHELDSFX is a set of C commands to execute after calling ncache_load().
  */
 
 #define	HASNCACHE	1
 /* #define	HASNCAPID	1	*/
+/* #define	NCACHELDPFX	??? */
+/* #define	NCACHELDSFX	??? */
 
 
 /*
@@ -374,6 +380,24 @@
 #define	HASSNODE	1
 
 
+/*
+ * Define HASSPECDEVD to be the name of a function that handles the results
+ * of a successful stat(2) of a file name argument.
+ *
+ * For example, HASSPECDEVD() for Darwin makes sure that st_dev is set to
+ * what stat("/dev") returns -- i.e., what's in DevDev.
+ *
+ * The function takes two arguments:
+ *
+ *	1: pointer to the full path name of file
+ *	2: pointer to the stat(2) result
+ *
+ * The function returns void.
+ */
+
+/* #define	HASSPECDEVD	process_dev_stat */
+
+
 /*
  * HASSTREAMS is defined for those dialects that support streams.
  */
@@ -458,6 +482,16 @@
 
 /* #define	USE_LIB_RNMH			1	   rnmh.c */
 
+# if	PTXV<450
+#define	USE_LIB_SNPF				1	/* snpf.c */
+#  if	PTXV<400
+#define	HAS_NO_LONG_LONG			1	/* disable snpf()'s
+							 * long long support */
+#  endif	/* PTXV<400 */
+# else	/* PTXV>=450 */
+#define	snpf	snprintf	   /* use the system's snprintf() */
+# endif	/* PTXV<450 */
+
 
 /*
  * WARNDEVACCESS is defined for those dialects that should issue a warning
