@@ -1,5 +1,5 @@
 /*
- * dlsof.h - SunOS (Solaris 1.x and 2.x) header file for lsof
+ * dlsof.h - Solaris header file for lsof
  */
 
 
@@ -31,28 +31,22 @@
 
 
 /*
- * $Id: dlsof.h,v 1.25 2000/06/25 17:31:11 abe Exp $
+ * $Id: dlsof.h,v 1.27 2000/12/04 14:35:13 abe Exp $
  */
 
 
-#if	!defined(SUN_LSOF_H)
-#define	SUN_LSOF_H	1
+#if	!defined(SOLARIS_LSOF_H)
+#define	SOLARIS_LSOF_H	1
 
-# if	defined(solaris)
 #include <fcntl.h>
 #include <sys/mntent.h>
 #include <sys/mnttab.h>
 
-#  if	solaris<20600
+# if	solaris<20600
 #define	_KMEMUSER	1
-#  else	/* solaris>=20600 */
+# else	/* solaris>=20600 */
 #include <stddef.h>
-#  endif	/* solaris<20600 */
-
-# else		/* not solaris */
-#include <mntent.h>
-#include <sys/domain.h>
-# endif	/* solaris */
+# endif	/* solaris<20600 */
 
 #include <stdlib.h>
 #include <dirent.h>
@@ -65,40 +59,34 @@
 #include <rpc/types.h>
 #include <sys/protosw.h>
 #include <sys/socket.h>
-
-# if	!defined(solaris)
-#include <sys/socketvar.h>
-# endif	/* !defined(solaris) */
-
 #include <sys/un.h>
 #include <sys/wait.h>
 #include <netinet/in.h>
 
-# if	defined(solaris)
-#  if	solaris>=70000
+# if	solaris>=70000
 #include <sys/conf.h>
 #include <sys/systeminfo.h>
-#  endif	/* solaris>=70000 */
+# endif	/* solaris>=70000 */
 
 #define	_KERNEL
 #define	MI_HRTIMING
 #include <inet/led.h>
 
-#  if	solaris<20600
+# if	solaris<20600
 #undef	staticf
-#  endif	/* solaris<20600 */
+# endif	/* solaris<20600 */
 
 #include <inet/common.h>
 
-#  if	solaris>=70000
+# if	solaris>=70000
 #include <sys/stropts.h>
-#  endif	/* solaris>=70000 */
+# endif	/* solaris>=70000 */
 
 #include <inet/mi.h>
 
-#  if	solaris>=80000
+# if	solaris>=80000
 #include <netinet/ip6.h>
-#  endif	/* solaris>=80000 */
+# endif	/* solaris>=80000 */
 
 #include <inet/ip.h>
 #undef	_KERNEL
@@ -112,55 +100,45 @@
 #undef	rval_t
 #undef	strsignal
 
-#  if	solaris>=80000
+# if	solaris>=80000
 #define	_KERNEL	1
-#  endif	/* solaris>=80000 */
+# endif	/* solaris>=80000 */
 
 #include <inet/tcp.h>
 
-#  if	solaris>=80000
+# if	solaris>=80000
 #undef	_KERNEL
-#  endif	/* solaris>=80000 */
-
-# else	/* !solaris */
-#include <netinet/tcp.h>
-#include <netinet/tcp_fsm.h>
-#include <netinet/tcp_timer.h>
-#include <netinet/tcp_var.h>
-#include <sys/unpcb.h>
-# endif	/*  solaris */
+# endif	/* solaris>=80000 */
 
 #include <net/route.h>
 #include <netinet/in_pcb.h>
 #include <sys/stream.h>
 
-# if	defined(solaris) && solaris<20600
+# if	solaris<20600
 #undef	MAX
 #undef	MIN
-# endif	/* defined(solaris) && solaris<20600 */
+# endif	/* solaris<20600 */
 
 #include <sys/sysmacros.h>
 #include <sys/vfs.h>
 #include <sys/vnode.h>
-
-# if	defined(solaris)
 #include <sys/fs/hsfs_spec.h>
 #include <sys/fs/hsfs_node.h>
 #include <sys/fs/lofs_node.h>
 
-#  if	solaris>=20600
+# if	solaris>=20600
 #define	_KERNEL
-#  endif	/* solaris>=20600 */
+# endif	/* solaris>=20600 */
 
 #include <sys/fs/namenode.h>
 
-#  if	solaris>=20600
+# if	solaris>=20600
 #undef	_KERNEL
-#  endif	/* solaris>=20600 */
+# endif	/* solaris>=20600 */
 
 #include <sys/tihdr.h>
 
-#  if	solaris>=20500
+# if	solaris>=20500
 #include <sys/tiuser.h>
 #include <rpc/auth.h>
 #include <rpc/clnt.h>
@@ -170,55 +148,17 @@
 #include <sys/fs/autofs.h>
 #include <sys/door.h>
 #undef	_KERNEL
-#  else	/* solaris<20500 */
-#include <rpc/rpc.h>
-#include <rpc/clnt_soc.h>
-#include <rpc/pmap_prot.h>
-#  endif	/* solaris>=20500 */
+# endif	/* solaris>=20500 */
+
 #  if	!defined(_NETDB_H_)
 #include <rpc/rpcent.h>
 #  endif	/* !defined(_NETDB_H_) */
-# else	/* !solaris */
-#include <rpc/rpc.h>
-#include <rpc/pmap_prot.h>
 
-/*
- * Dummy hsnode structure for below SunOS 4.1.3.  This should come from
- * hsfs_node.h, but it has unresolvable protoype declaration conflicts.
- */
-
-#  if	SUNOSV<40103
-struct  hsnode {
-	struct hsnode	*dummy1[3];
-	struct vnode	dummy2;
-	struct	hs_direntry {
-		u_int	ext_lbn;	/* SunOS 4.1.x inode number */
-		u_int	ext_size;    	/* size */
-	} hs_dirent;
-};
-#  else	/* SUNOSV>=40103 */
-#include <hsfs/hsfs_spec.h>
-struct cred;
-#define	strcpy	LSOF_strcpy
-#define	strncpy	LSOF_strncpy
-
-#   if	defined(HAS_HSFS_NODE)
-#include <hsfs/hsfs_node.h>
-#   else	/* !defined(HAS_HSFS_NODE) */
-#include <hsfs/hsnode.h>
-#   endif	/* defined(HAS_HSFS_NODE) */
-
-#undef	strcpy
-#undef	strncpy
-#  endif	/* SUNOSV<40103 */
-# endif	/* solaris */
-
-# if	defined(solaris)
 #include <sys/t_lock.h>
 #include <sys/flock.h>
 
-#  if	solaris>=20300
-#   if	solaris<20400
+# if	solaris>=20300
+#  if	solaris<20400
 /*
  * The lock_descriptor structure definition is missing from Solaris 2.3.
  */
@@ -246,10 +186,10 @@ struct lock_descriptor {
 	} info;
 };
 #define	ACTIVE_LOCK	0x008		/* lock is active */
-#   else	/* solaris>=20400 */
+#  else	/* solaris>=20400 */
 #include <sys/flock_impl.h>
-#   endif	/* solaris<20400 */
-#  endif	/* solaris>=20300 */
+#  endif	/* solaris<20400 */
+# endif	/* solaris>=20300 */
 
 #include <sys/fstyp.h>
 #include <sys/dditypes.h>
@@ -263,23 +203,12 @@ struct lock_descriptor {
 #include <sys/fs/pc_label.h>
 #include <sys/fs/pc_node.h>
 
-#  if	solaris>=20600
+# if	solaris>=20600
 #undef	SLOCKED
-#  endif	/* solaris>=20600 */
+# endif	/* solaris>=20600 */
 
 #include <sys/fs/snode.h>
 #include <sys/fs/tmpnode.h>
-# else	/* !solaris */
-#include <net/raw_cb.h>
-#include <ufs/lockf.h>
-#include <ufs/inode.h>
-#include <specfs/snode.h>
-#include <tmpfs/tmpnode.h>
-#include <sys/mbuf.h>
-#include <sys/domain.h>
-#include <ufs/mount.h>
-# endif	/* solaris */
-
 #include <nfs/nfs.h>
 #include <nfs/rnode.h>
 #include <sys/proc.h>
@@ -289,19 +218,6 @@ struct lock_descriptor {
 #include <sys/proc/prdata.h>
 # endif	/* defined(HASPROCFS) */
 
-
-# if	!defined(solaris)
-#  if	!defined(KERNEL)
-#define KERNEL	1
-#  endif
-#include <sys/file.h>
-#include <pcfs/pc_dir.h>
-#include <pcfs/pc_label.h>
-#include <pcfs/pc_node.h>
-#include <specfs/fifonode.h>
-#undef	KERNEL
-# endif	/* solaris */
-
 #include <sys/file.h>
 #include <vm/hat.h>
 #include <vm/as.h>
@@ -309,12 +225,9 @@ struct lock_descriptor {
 #include <vm/seg_dev.h>
 #include <vm/seg_map.h>
 #include <vm/seg_vn.h>
-
-# if	defined(solaris)
 #include <sys/tiuser.h>
 #include <sys/t_kuser.h>
 #include <sys/sockmod.h>
-# endif	/* defined(solaris) */
 
 /*
  * Structure for Atria's MVFS nodes
@@ -327,40 +240,26 @@ struct mvfsnode {
 
 extern int nlist();
 
-# if	!defined(solaris)
-extern char *sys_errlist[];
-#define	strerror(errno)	sys_errlist[errno]
-# endif
-
 # if	defined(HAS_AFS) && !defined(AFSAPATHDEF)
 #define	AFSAPATHDEF	"/usr/vice/etc/modload/libafs"
 # endif	/* defined(HAS_AFS) && !defined(AFSAPATHDEF) */
 
-# if	defined(NOCONST)
-#define	COMP_P		void
-# else
 #define	COMP_P		const void
-# endif
-
 #define	CWDLEN		(MAXPATHLEN+1)
 #define DEVINCR		1024		/* device table malloc() increment */
 #define	DIRTYPE		dirent
 
-# if	defined(solaris)
-#  if	solaris>=70000
+# if	solaris>=70000
 typedef	uintptr_t	KA_T;
-#  else	/* solaris<70000 */
+# else	/* solaris<70000 */
 typedef	void *		KA_T;
-#  endif	/* solaris>=70000 */
-# else	/* !defined(solaris) */
-typedef	unsigned long	KA_T;
-# endif	/* defined(solaris) */
+# endif	/* solaris>=70000 */
 
-# if	defined(solaris) && solaris>=20501
+# if	solaris>=20501
 #define	KMEM		"/dev/mem"
-# else	/* !defined(solaris) || solaris<20501 */
+# else	/* solaris<20501 */
 #define	KMEM		"/dev/kmem"
-# endif	/* defined(solaris) && solaris>=20501 */
+# endif	/* solaris>=20501 */
 
 #define MALLOC_P	char
 #define FREE_P		MALLOC_P
@@ -371,13 +270,11 @@ typedef	unsigned long	KA_T;
 # endif	/* !defined(MAXEND) */
 
 #define MAXSEGS		100		/* maximum text segments */
-
-# if	defined(solaris)
 #define	DINAMEL		32
 
-#  if	solaris>=70000
+# if	solaris>=70000
 #define	KA_T_FMT_X	"0x%p"
-#  endif	/* solaris>=70000 */
+# endif	/* solaris>=70000 */
 
 #define NETCLNML	8
 #define	N_UNIX		"/dev/ksyms"
@@ -387,14 +284,6 @@ typedef	unsigned long	KA_T;
 #define	PR_ROOTINO	2		/* root inode for proc file system */
 #  endif	/* defined(HASPROCFS) */
 
-# else	/* !defined(solaris) */
-#define CLONEMAJ	37		/* major device number for clone
-					 * device */
-#define	MEMMOVE		memcpy		/* for fill_portmap() in print.c */
-#define N_UNIX		"/vmunix"
-#define	PROCMIN		3		/* processes that make a "good" scan */
-# endif	/* defined(solaris) */
-
 #define	PROCDFLT	256		/* default size for local proc table */
 #define PROCSIZE	sizeof(struct proc)
 #define	PROCTRYLM	5		/* times to try to read proc table */
@@ -403,13 +292,13 @@ typedef	unsigned long	KA_T;
 #define STRNCPY_L	int
 #define	STRNML		32		/* stream name length (maximum) */
 
-# if	defined(solaris) && solaris>=20501
+# if	solaris>=20501
 #define	SZOFFTYPE	unsigned long long
 					/* size and offset internal storage
 					 * type */
 #define	SZOFFPSPEC	"ll"		/* SZOFFTYPE print specification
 					 * modifier */
-# endif	/* defined(solaris) && solaris>=20600 */
+# endif	/* solaris>=20501 */
 
 #define U_SIZE		sizeof(struct user)
 
@@ -428,11 +317,7 @@ extern char *AFSApath;			/* alternate AFS name list path
 extern dev_t AFSdev;			/* AFS file system device number */
 extern int AFSdevStat;			/* AFS file system device number
 					 * status: 0 = unknown; 1 = known */
-
-#  if	defined(solaris)
 extern int AFSfstype;			/* AFS file system type index */
-#  endif	/* defined(solaris) */
-
 extern KA_T AFSVfsp;			/* AFS struct vfs kernel pointer */
 # endif	/* defined(HAS_AFS) */
 
@@ -443,16 +328,9 @@ struct clone {
 };
 extern struct clone *Clone;
 
-# if	defined(solaris)
 extern	major_t	CloneMaj;
 extern char **Fsinfo;
 extern int Fsinfomax;
-# else	/* !defined(solaris) */
-extern char *Cwd;
-extern struct file *Fileptr;
-#define	FILEPTR	Fileptr			/* for process_file() in dfile.c */
-# endif	/* defined(solaris) */
-
 extern int HaveCloneMaj;
 extern kvm_t *Kd;
 
@@ -466,10 +344,10 @@ struct l_vfs {
 	ino_t fs_ino;			/* file system inode number */
 # endif	/* defined(HASFSINO) */
 
-# if	defined(solaris) && solaris>=80000
+# if	solaris>=80000
 	nlink_t nlink;			/* directory link count */
 	off_t size;			/* directory size */
-# endif	/* defined(solaris) && solaris>=80000 */
+# endif	/* solaris>=80000 */
 
 	struct l_vfs *next;		/* forward link */
 };
@@ -492,30 +370,25 @@ struct mounts {
 	char *fstype;			/* file system type */
 # endif	/& defined(HASFSTYPE) */
 
-# if	defined(solaris) && solaris>=80000
+# if	solaris>=80000
 	nlink_t nlink;			/* directory st_nlink */
 	off_t size;			/* directory st_size */
-# endif	/* defined(solaris) && solaris>=80000 */
+# endif	/* solaris>=80000 */
 
 };
 
-# if	defined(solaris)
 struct pseudo {
 	struct l_dev pd;		/* device, inode, path, verify */
 	struct pseudo *next;		/* forward link */
 };
 extern struct pseudo *Pseudo;
-# endif	/* solaris */
-
-# if	!defined(solaris)
-extern char *Rtd;
-# endif	/* !defined(solaris) */
 
 struct sfile {
 	char *aname;			/* file name argument */
 	char *name;			/* file name (after readlink()) */
 	char *devnm;			/* device name (optional) */
 	dev_t dev;			/* device */
+	dev_t rdev;			/* raw device */
 	u_short mode;			/* S_IFMT mode bits from stat() */
 	int type;			/* file type: 0 = file system
 				 	 *	      1 = regular file */
@@ -553,16 +426,10 @@ struct sfile {
 #define	DCACHE_CLR	clr_sect	/* function to clear clone and
 					 * pseudo caches when reading the
 					 * device cache file fails */
-#  if	defined(solaris)
 #define	DCACHE_PSEUDO	rw_pseudo_sect	/* pseudo function for read_dcache */
-#  endif	/* defined_solaris */
 # endif	/* defined(HASDCACHE) */
 
-# if	defined(solaris)
 #define	DVCH_DEVPATH	"/devices"
-# else	/* !defined(solaris) */
-#define	DVCH_DEVPATH	"/dev"
-# endif	/* defined(solaris) */
 
 
 /*
@@ -571,10 +438,10 @@ struct sfile {
 
 #define	CVFS_DEVSAVE 	1
 
-# if	defined(solaris) && solaris>=80000
+# if	solaris>=80000
 #define	CVFS_NLKSAVE	1
 #define	CVFS_SZSAVE	1
-# endif	/* defined(solaris) && solaris>=80000 */
+# endif	/* solaris>=80000 */
 
 
 /*
@@ -588,9 +455,9 @@ struct sfile {
 #define	HASDNLCPTR	1
 #  endif	/* !defined(NC_NAMLEN) */
 
-#  if	defined(solaris) && solaris>=80000
+#  if	solaris>=80000
 #define	NCACHE_NEGVN	"negative_cache_vnode"
-#  endif	/* defined(solaris) && solaris>=80000 */
+#  endif	/* solaris>=80000 */
 # endif	/* defined(HASNCACHE) */
 
-#endif	/* SUN_LSOF_H */
+#endif	/* SOLARIS_LSOF_H */
