@@ -1,5 +1,5 @@
 /*
- * machine.h - SunOS (Solaris 1.x and 2.x)  definitions for lsof
+ * machine.h - Solaris definitions for lsof
  */
 
 
@@ -34,7 +34,7 @@
 #define	LSOF_MACHINE_H	1
 
 
-# if	defined(solaris) && solaris>=20600
+# if	solaris>=20600
 /*
  * <sys/poll.h> must be #include'd for Solaris 2.6 while _KMEMUSER is
  * defined.  Since <netdb.h> also #include's <sys/poll.h> and <netdb.h>
@@ -56,11 +56,11 @@
 #define	ipa_32	s6_ipaddr.ipa_32
 # endif	/* solaris>=80000 */
 
-# endif	/* defined(solaris) && solaris>=20600 */
+# endif	/* solaris>=20600 */
 
 
 /*
- * $Id: machine.h,v 1.21 2000/03/16 15:56:27 abe Exp $
+ * $Id: machine.h,v 1.26 2000/12/05 20:28:07 abe Exp $
  */
 
 
@@ -69,9 +69,9 @@
  * can be used to obtain a CLIENT handle in lieu of clnttcp_create().
  */
 
-#if	!defined(solaris) || (defined(solaris) && solaris>=20501)
+#if	solaris>=20501
 #define	CAN_USE_CLNT_CREATE	1
-#endif	/* !defined(solaris) || (defined(solaris) && solaris>=20501) */
+#endif	/* solaris>=20501 */
 
 
 /*
@@ -79,9 +79,9 @@
  * getdtablesize() to obtain the maximum file descriptor number plus one.
  */
 
-#if	defined(solaris) && solaris<20500
+#if	solaris<20500
 #define	GET_MAX_FD	get_max_fd
-#endif	/* defined(solaris) && solaris<20500 */
+#endif	/* solaris<20500 */
 
 
 /*
@@ -186,9 +186,7 @@
  * use readinode() from node.c.
  */
 
-#if	!defined(solaris)
-#define	HASINODE	1
-#endif	/* !defined(solaris) */
+/* #define	HASINODE	1	*/
 
 
 /*
@@ -243,11 +241,19 @@
  *
  * HASNCAPID is defined for those dialects with a searchable kernel name
  * cache whose cache and vnodes are linked by a capability ID.
+ *
+ * NCACHELDPFX is a set of C commands to execute before calling ncache_load().
+ *
+ * NCACHELDSFX is a set of C commands to execute after calling ncache_load().
  */
 
 #define	HASNCACHE	1
 /* #define	HASNCAPID	1	*/
 
+#define	NCACHELDPFX	open_kvm();	/* do before calling ncache_load() */
+
+/* #define	NCACHELDSFX	??? */
+
 
 /*
  * HASNLIST is defined for those dialects that use nlist() to acccess
@@ -298,12 +304,7 @@
  * called from print_file().
  */
 
-#if	defined(solaris)
 #define	HASPRINTDEV	print_dev
-#else	/* !defined(solaris) */
-#define	HASPRINTINO	print_ino
-#endif	/* defined(solaris) */
-
 /* #define	HASPRINTNM	print_nm?	*/
 /* #define	HASPRINTOFF	print_off?	*/
 /* #define	HASPRINTSZ	print_sz?	*/
@@ -368,11 +369,9 @@
  * by inode number.
  */
 
-# if	defined(solaris)
 #define	HASPROCFS	"proc"
 #define	HASFSTYPE	1
 #define	HASPINODEN	1
-# endif	/* defined(solaris) */
 
 
 /*
@@ -399,6 +398,24 @@
 #define	HASSNODE	1
 
 
+/*
+ * Define HASSPECDEVD to be the name of a function that handles the results
+ * of a successful stat(2) of a file name argument.
+ *
+ * For example, HASSPECDEVD() for Darwin makes sure that st_dev is set to
+ * what stat("/dev") returns -- i.e., what's in DevDev.
+ *
+ * The function takes two arguments:
+ *
+ *	1: pointer to the full path name of file
+ *	2: pointer to the stat(2) result
+ *
+ * The function returns void.
+ */
+
+/* #define	HASSPECDEVD	process_dev_stat */
+
+
 /*
  * HASSTREAMS is defined for those dialects that support streams.
  */
@@ -411,9 +428,9 @@
  * TCP/TPI Recv-Q and Send-Q values produced by netstat.
  */
 
-#if	!defined(solaris) || solaris==20300 || solaris>=20500
+#if	solaris==20300 || solaris>=20500
 #define	HASTCPTPIQ	1
-#endif	/* !defined(solaris) || solaris==20300 || solaris>=20500 */
+#endif	/* solaris==20300 || solaris>=20500 */
 
 
 /*
@@ -421,9 +438,9 @@
  * TCP/TPI send and receive window sizes produced by netstat.
  */
 
-#if	defined(solaris) && (solaris==20300 || solaris>=20500)
+#if	solaris==20300 || solaris>=20500
 #define	HASTCPTPIW	1
-#endif	/* defined(solaris) && (solaris==20300 || solaris>=20500) */
+#endif	/* solaris==20300 || solaris>=20500 */
 
 
 /*
@@ -476,17 +493,19 @@
 #define	USE_LIB_LKUPDEV				1	/* lkud.c */
 /* #define	USE_LIB_PRINTCHDEVNAME		1	   pdvn.c */
 /* #define	USE_LIB_PROCESS_FILE		1	   prfp.c */
-
-#if	!defined(solaris)
-#define	USE_LIB_PRINT_TCPTPI			1	/* ptti.c */
-#endif	/* !defined(solaris) */
-
+/* #define	USE_LIB_PRINT_TCPTPI		1	   ptti.c */
 /* #define	USE_LIB_READDEV			1	   rdev.c */
 /* #define	USE_LIB_READMNT			1	   rmnt.c */
 /* #define	USE_LIB_RNAM			1	   rnam.c */
 #define	USE_LIB_RNCH				1	/* rnch.c */
 /* #define	USE_LIB_RNMH			1	   rnmh.c */
 
+# if	solaris<20600
+#define	USE_LIB_SNPF				1	/* snpf.c */
+# else	/* solaris>=20600 */
+#define	snpf	snprintf	   /* use the system's snprintf() */
+# endif	/* solaris<20600 */
+
 
 /*
  * WARNDEVACCESS is defined for those dialects that should issue a warning
@@ -518,10 +537,6 @@
  * zeromem is a macro that uses bzero or memset.
  */
 
-# if	defined(solaris)
 #define	zeromem(a, l)		memset((void *)a, 0, l)
-# else	/* !defined(solaris) */
-#define	zeromem(a, l)		memset((char *)a, 0, l)
-# endif	/* defined(solaris) */
 
 #endif	/* !defined(LSOF_MACHINE_H) */
