@@ -32,20 +32,30 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1996 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dnode.c,v 1.5 98/02/16 19:56:03 abe Exp $";
+static char *rcsid = "$Id: dnode.c,v 1.18 2000/12/06 14:31:08 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 #include <sys/fs/namenode.h>
+
+#if	UNIXWAREV>=70000
+#undef	IREAD
+#undef	IWRITE
+#undef	IEXEC
+#endif	/* UNIXWAREV>=70000 */
+
 #include <sys/fs/ufs_inode.h>
+
+#if	defined(HASXNAMNODE)
 #include <sys/fs/xnamnode.h>
+#endif	/* defined(HASXNAMNODE) */
 
 
-_PROTOTYPE(static void ent_fa,(caddr_t *a1, caddr_t *a2, char *d));
-_PROTOTYPE(static int get_vty,(struct vnode *v, caddr_t va, struct vfs *kv, int *fx));
-_PROTOTYPE(static int examine_stream,(struct stdata *vs, struct queue *q, char *mn, char *sn, KA_T *sqp));
-_PROTOTYPE(static struct l_dev * findstrdev,(dev_t *dev));
+_PROTOTYPE(static void ent_fa,(KA_T *a1, KA_T *a2, char *d));
+_PROTOTYPE(static int get_vty,(struct vnode *v, KA_T va, struct vfs *kv, int *fx));
+_PROTOTYPE(static int examine_stream,(KA_T vs, struct queue *q, char *mn, char *sn, KA_T *sqp));
+_PROTOTYPE(static struct l_dev * findstrdev,(dev_t *dev, dev_t *rdev));
 _PROTOTYPE(static char isvlocked,(struct vnode *va));
 _PROTOTYPE(static int readlino,(int fx, struct vnode *v, struct l_ino *i));
 
@@ -74,19 +84,22 @@ static struct protos {
 
 static void
 ent_fa(a1, a2, d)
-	caddr_t *a1;			/* first fattach address (NULL OK) */
-	caddr_t *a2;			/* second fattach address */
+	KA_T *a1;			/* first fattach address (NULL OK) */
+	KA_T *a2;			/* second fattach address */
 	char *d;			/* direction ("->" or "<-") */
 {
-	char buf[64], *cp;
+	char buf[64], *cp, tbuf[32];
 	MALLOC_S len;
 
 	if (Lf->nma)
-		return;
+	    return;
 	if (!a1)
-	    (void) sprintf(buf, "(FA:%s%#x)", d, *a2);
+	    (void) snpf(buf, sizeof(buf), "(FA:%s%s)", d,
+		print_kptr(*a2, (char *)NULL, 0));
 	else
-	    (void) sprintf(buf, " (FA:%#x%s%#x)", *a1, d, *a2);
+	    (void) snpf(buf, sizeof(buf), " (FA:%s%s%s)",
+		print_kptr(*a1, tbuf, sizeof(tbuf)), d,
+		print_kptr(*a2, (char *)NULL, 0));
 	len = strlen(buf) + 1;
 	if ((cp = (char *)malloc(len)) == NULL) {
 	    (void) fprintf(stderr,
@@ -94,7 +107,7 @@ ent_fa(a1, a2, d)
 		Pn, Lp->pid, Lf->fd);
 	    Exit(1);
 	}
-	(void) strcpy(cp, buf);
+	(void) snpf(cp, len, "%s", buf);
 	Lf->nma = cp;
 }
 
@@ -105,21 +118,24 @@ ent_fa(a1, a2, d)
 
 static int
 examine_stream(vs, q, mn, sn, sqp)
-	struct stdata *vs;		/* stream head's stdata pointer */
+	KA_T vs;			/* stream head's stdata kernel
+					 * address */
 	struct queue *q;		/* queue structure buffer */
 	char *mn;			/* module name receiver */
 	char *sn;			/* special module name */
 	KA_T *sqp;			/* special module's q_ptr */
 {
 	struct module_info mi;
-	struct queue *qp;
+	KA_T qp;
 	struct qinit qi;
 	struct stdata sd;
+	char tbuf[32];
 /*
  * Read stream's head.
  */
 	if (!vs || readstdata(vs, &sd)) {
-	    (void) sprintf(Namech, "can't read stream head from %x", vs);
+	    (void) snpf(Namech, Namechl, "can't read stream head from %s",
+		print_kptr(vs, (char *)NULL, 0));
 	    enter_nm(Namech);
 	    return(1);
 	}
@@ -130,31 +146,35 @@ examine_stream(vs, q, mn, sn, sqp)
 /*
  * Examine the write queue.
  */
-	for (qp = sd.sd_wrq, *mn = '\0'; qp; qp = q->q_next) {
+	for (qp = (KA_T)sd.sd_wrq, *mn = '\0'; qp; qp = (KA_T)q->q_next) {
 
 	/*
 	 * Read stream queue entry.
 	 */
-	    if (kread((KA_T)qp, (char *)q, sizeof(struct queue))) {
-		(void) sprintf(Namech, "can't read stream queue from %x", qp);
+	    if (kread(qp, (char *)q, sizeof(struct queue))) {
+		(void) snpf(Namech, Namechl, "can't read stream queue from %s",
+		    print_kptr(qp, (char *)NULL, 0));
 		enter_nm(Namech);
 		return(1);
 	    }
 	/*
 	 * Read queue's information structure.
 	 */
-	    if (!q->q_qinfo || readstqinit(q->q_qinfo, &qi)) {
-		(void) sprintf(Namech, "can't read qinit for %x from %x",
-		    qp, q->q_qinfo);
+	    if (!q->q_qinfo || readstqinit((KA_T)q->q_qinfo, &qi)) {
+		(void) snpf(Namech, Namechl, "can't read qinit for %s from %s",
+		    print_kptr(qp, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)q->q_qinfo, (char *)NULL, 0));
 		enter_nm(Namech);
 		return(1);
 	    }
 	/*
 	 * Read module information structure.
 	 */
-	    if (!qi.qi_minfo || readstmin(qi.qi_minfo, &mi)) {
-		(void) sprintf(Namech, "can't read module info for %x from %x",
-		    q->q_qinfo, qi.qi_minfo);
+	    if (!qi.qi_minfo || readstmin((KA_T)qi.qi_minfo, &mi)) {
+		(void) snpf(Namech, Namechl,
+		    "can't read module info for %s from %s",
+		    print_kptr((KA_T)q->q_qinfo, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)qi.qi_minfo, (char *)NULL, 0));
 		enter_nm(Namech);
 		return(1);
 	    }
@@ -162,8 +182,10 @@ examine_stream(vs, q, mn, sn, sqp)
 	 * Read module name.
 	 */
 	    if (!mi.mi_idname || kread((KA_T)mi.mi_idname, mn, STRNML-1)) {
-		(void) sprintf(Namech, "can't read module name for %x from %x",
-		    qi.qi_minfo, mi.mi_idname);
+		(void) snpf(Namech, Namechl,
+		    "can't read module name for %s from %s",
+		    print_kptr((KA_T)qi.qi_minfo, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)mi.mi_idname, (char *)NULL, 0));
 		enter_nm(Namech);
 		return(1);
 	    }
@@ -185,8 +207,9 @@ examine_stream(vs, q, mn, sn, sqp)
  */
 
 static struct l_dev *
-findstrdev(dev)
+findstrdev(dev, rdev)
 	dev_t *dev;			/* device */
+	dev_t *rdev;			/* raw device */
 {
 	struct clone *c;
 	struct l_dev *dp;
@@ -200,14 +223,14 @@ findstrdev_again:
 
 #endif	/* HASDCACHE */
 
-	if ((dp = lkupdev(dev, 0, 0)))
+	if ((dp = lkupdev(dev, rdev, 0, 0)))
 	    return(dp);
 /*
  * Search for clone.
  */
 	if (Clone) {
 	    for (c = Clone; c; c = c->next) {
-		if (major(*dev) == minor(Devtp[c->dx].rdev)) {
+		if (major(*rdev) == minor(Devtp[c->dx].rdev)) {
 
 #if	HASDCACHE
 		    if (DCunsafe && !Devtp[c->dx].v && !vfy_dev(&Devtp[c->dx]))
@@ -225,51 +248,66 @@ findstrdev_again:
 /*
  * get_vty() - get vnode type
  *
- * return: vnode type as an N_* symbol value (-1 if unrecognized)
+ * return: vnode type as an N_* symbol value
+ *	   N_REGLR if no special file system type applies
+ *	   -1 if the vnode type is VUNNAMED
+ *	   -2 if the vfs structure has an illegal type index
+ *	   -3 if the vfs structure can't be read
  */
 
 static int
 get_vty(v, va, kv, fx)
 	struct vnode *v;		/* vnode to test */
-	caddr_t va;			/* vnode's kernel address */
-	struct vfs *kv;			/* vnode's kernel vfs struct */
+	KA_T va;			/* vnode's kernel address */
+	struct vfs *kv;			/* copy of vnode's kernel vfs struct */
 	int *fx;			/* file system type index */
 {
 	int fxt;
 	int nty = N_REGLR;
+	char tbuf[32];
 
-	if (!v->v_vfsp)
+	if (v->v_type == VUNNAMED) {
+	    *fx = 0;
+	    return(-1);
+	}
+	if (!v->v_vfsp) {
+	    *fx = 0;
+	    if (v->v_type == VFIFO && v->v_stream)
+		return(N_STREAM);
 	    return(N_REGLR);
-	if (kread((KA_T)v->v_vfsp, (char *)kv, sizeof(struct vfs)) == 0) {
+	}
+	if (!kread((KA_T)v->v_vfsp, (char *)kv, sizeof(struct vfs))) {
 
 	/*
 	 * Check the file system type.
 	 */
 	    fxt = kv->vfs_fstype;
 	    if (fxt > 0 && fxt <= Fsinfomax) {
-		if (strcmp(Fsinfo[fxt-1], "fifofs") == 0)
+		if (!strcmp(Fsinfo[fxt-1], "fifofs"))
 		    nty = N_FIFO;
-		else if (strcmp(Fsinfo[fxt-1], "nfs") == 0)
+		else if (!strcmp(Fsinfo[fxt-1], "nfs"))
 		    nty = N_NFS;
-		else if (strcmp(Fsinfo[fxt-1], "namefs") == 0)
+		else if (!strcmp(Fsinfo[fxt-1], "namefs"))
 		    nty = N_NM;
 
 #if	defined(HASPROCFS)
-		else if (strcmp(Fsinfo[fxt-1], "proc") == 0)
+		else if (!strcmp(Fsinfo[fxt-1], "proc"))
 		    nty = N_PROC;
 #endif	/* defined(HASPROCFS) */
 
 	    } else {
-		(void) sprintf(Namech,
-		    "vnode@%#x: bad file system index (%d)", Pn, va, fxt);
+		(void) snpf(Namech, Namechl,
+		    "vnode@%s: bad file system index (%d)",
+		    print_kptr(va, (char *)NULL, 0), fxt);
 		enter_nm(Namech);
-		return(-1);
+		return(-2);
 	    }
 	} else {
-	    (void) sprintf(Namech, "vnode@%#x: bad vfs pointer (%#x)",
-		va, v->v_vfsp);
+	    (void) snpf(Namech, Namechl, "vnode@%s: bad vfs pointer (%s)",
+		print_kptr(va, tbuf, sizeof(tbuf)),
+		print_kptr((KA_T)v->v_vfsp, (char *)NULL, 0));
 	    enter_nm(Namech);
-	    return(-1);
+	    return(-3);
 	}
 	if (nty == N_REGLR) {
 	    if (v->v_type == VFIFO)
@@ -290,17 +328,18 @@ static char
 isvlocked(va)
 	struct vnode *va;		/* local vnode address */
 {
-	struct filock f, *flf, *flp;
+	struct filock f;
+	KA_T flf, flp;
 	int i, l;
 
-	if (!(flf = va->v_filocks))
+	if (!(flf = (KA_T)va->v_filocks))
 	    return(' ');
 	flp = flf;
 	i = 0;
 	do {
 	    if (i++ > 1000)
 		break;
-	    if (kread((KA_T)flp, (char *)&f, sizeof(f)))
+	    if (kread(flp, (char *)&f, sizeof(f)))
 		break;
 	    if (f.set.l_sysid || f.set.l_pid != (pid_t)Lp->pid)
 		continue;
@@ -319,7 +358,7 @@ isvlocked(va)
 	    default:
 		return(' ');
 	    }
-	} while (flp != f.next && (flp = f.next) && flp != flf);
+	} while (flp != (KA_T)f.next && (flp = (KA_T)f.next) && flp != flf);
 	return(' ');
 }
 
@@ -330,22 +369,24 @@ isvlocked(va)
 
 void
 process_node(na)
-	caddr_t na;			/* vnode kernel space address */
+	KA_T na;			/* vnode kernel space address */
 {
-	char *cp;
-	dev_t dev;
-	char dev_ch[32];
+	char *cp, *ep;
+	dev_t dev, rdev;
+	unsigned char devs = 0;
+	unsigned char ni = 0;
+	unsigned char rdevs = 0;
 	struct l_dev *dp;
 	struct fifonode f;
 	int fx, rfx;
 	struct l_ino i;
+	int is = 1;
 	int j, k;
 	struct vfs kv, rkv;
-	struct vfs *la = NULL;
 	struct module_info mi;
 	char mn[STRNML];
 	struct mnode mno;
-	int ni = 0;
+	MALLOC_S msz;
 	struct namenode nn;
 	int px;
 	struct queue q;
@@ -356,10 +397,10 @@ process_node(na)
 	unsigned char sd = 1;
 	struct so_so so;
 	KA_T sqp = (KA_T)NULL;
-	char *ty;
+	size_t sz;
+	char tbuf[32], *ty;
 	enum vtype type;
 	struct sockaddr_un ua;
-	struct l_vfs *lvfs;
 
 #if	defined(HASPROCFS)
 	struct as as;
@@ -382,20 +423,28 @@ process_node(na)
 	    enter_nm("no vnode address");
 	    return;
 	}
-	if (readvnode((caddr_t)na, &v)) {
+	if (readvnode((KA_T)na, &v)) {
             enter_nm(Namech);
             return;
         }
 
-# if    defined(HASNCACHE)
-	Lf->na = (unsigned long)na;
-# endif /* defined(HASNCACHE) */
+#if	defined(HASNCACHE)
+	Lf->na = na;
+#endif	/* defined(HASNCACHE) */
+
+#if	defined(HASFSTRUCT)
+	Lf->fna = na;
+	Lf->fsv |= FSV_NI;
+#endif	/* defined(HASFSTRUCT) */
 
 /*
  * Determine the vnode type.
  */
-	if ((Ntype = get_vty(&v, na, &kv, &fx)) < 0)
-		return;
+	if ((Ntype = get_vty(&v, na, &kv, &fx)) < 0) {
+	    if (Ntype == -1)
+		Lf->sf = 0;
+	    return;
+	}
 /*
  * Determine the lock state.
  */
@@ -403,41 +452,38 @@ process_node(na)
 get_lock_state:
 
 	Lf->lock = isvlocked(&v);
-/*
- * Establish the local virtual file system structure.
- */
-	if (!v.v_vfsp)
-	    lvfs = (struct l_vfs *)NULL;
-	else {
-	    if ((lvfs = readvfs(v.v_vfsp, &kv)) == NULL) {
-		(void) sprintf(Namech, "bad vfs for %#x at %#x", na, v.v_vfsp);
-		enter_nm(Namech);
-		return;
-	    }
-	}
 /*
  * Read the fifonode, inode, namenode, prnode, rnode, snode, ...
  */
 	switch (Ntype) {
 	case N_FIFO:
-	    if (!v.v_data || readfifonode(v.v_data, &f)) {
-		(void) sprintf(Namech,
-		    "vnode@%#x: can't read fifonode (%#x)", na, v.v_data);
+	    if (!v.v_data || readfifonode((KA_T)v.v_data, &f)) {
+		(void) snpf(Namech, Namechl,
+		    "vnode@%s: can't read fifonode (%s)",
+		    print_kptr(na, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
 	    if (f.fn_realvp) {
-		if (readvnode((caddr_t)f.fn_realvp, &rv)) {
-		    (void) sprintf(Namech,
-			"fifonode@%#x: can't read real vnode (%#x)",
-			v.v_data, f.fn_realvp);
+		if (readvnode((KA_T)f.fn_realvp, &rv)) {
+		    (void) snpf(Namech, Namechl,
+			"fifonode@%s: can't read real vnode (%s)",
+			print_kptr((KA_T)v.v_data, tbuf, sizeof(tbuf)),
+			print_kptr((KA_T)f.fn_realvp, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
-		if (!rv.v_data || readlino(fx, &rv, &i)) {
-		    (void) sprintf(Namech,
-			"fifonode@%#x: can't read inode (%#x)",
-			v.v_data, rv.v_data);
+
+#if	defined(HASNCACHE)
+		Lf->na = (KA_T)f.fn_realvp;
+#endif	/* defined(HASNCACHE) */
+
+		if (!rv.v_data || (is = readlino(fx, &rv, &i))) {
+		    (void) snpf(Namech, Namechl,
+			"fifonode@%s: can't read inode (%s)",
+			print_kptr((KA_T)v.v_data, tbuf, sizeof(tbuf)),
+			print_kptr((KA_T)rv.v_data, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
@@ -445,17 +491,19 @@ get_lock_state:
 		ni = 1;
 	    break;
 	case N_NFS:
-	    if (!v.v_data || readrnode((caddr_t)v.v_data, &r)) {
-		(void) sprintf(Namech, "vnode@%#x: can't read rnode (%#x)",
-		    na, v.v_data);
+	    if (!v.v_data || readrnode((KA_T)v.v_data, &r)) {
+		(void) snpf(Namech, Namechl, "vnode@%s: can't read rnode (%s)",
+		    print_kptr(na, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
 	    break;
 	case N_NM:
 	    if (!v.v_data || kread((KA_T)v.v_data, (char *)&nn, sizeof(nn))) {
-		(void) sprintf(Namech, "vnode@%#x: no namenode (%#x)",
-		    na, v.v_data);
+		(void) snpf(Namech, Namechl, "vnode@%s: no namenode (%s)",
+		    print_kptr(na, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -468,15 +516,19 @@ get_lock_state:
 	/*
 	 * The name node is mounted over/to another vnode.  Process that node.
 	 */
-	    (void) ent_fa(&na, (caddr_t *)nn.nm_mountpt, "->");
+	    (void) ent_fa(&na, (KA_T *)&nn.nm_mountpt, "->");
 	    if (kread((KA_T)nn.nm_mountpt, (char *)&rv, sizeof(rv))) {
-		(void) sprintf(Namech,
-		    "vnode@%#x: can't read namenode's mounted vnode (%#x)",
-		    na, nn.nm_mountpt);
+		(void) snpf(Namech, Namechl,
+		    "vnode@%s: can't read namenode's mounted vnode (%s)",
+		    print_kptr(na, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)nn.nm_mountpt, (char *)NULL, 0));
 		return;
 	    }
-	    if ((Ntype = get_vty(&rv, (caddr_t)nn.nm_mountpt, &rkv, &rfx)) < 0)
+	    if ((Ntype = get_vty(&rv, (KA_T)nn.nm_mountpt, &rkv, &rfx)) < 0) {
+		if (Ntype == -1)
+		    Lf->sf = 0;
 		return;
+	    }
 	/*
 	 * Unless the mounted-over/to node is another "namefs" node, promote
 	 * it to the vnode of interest.
@@ -486,18 +538,15 @@ get_lock_state:
 	    fx = rfx;
 	    kv = rkv;
 	    v = rv;
-	    if (lvfs) {
-		(void) free((MALLOC_P *)lvfs);
-		lvfs = (struct l_vfs *)NULL;
-	    }
 	    goto get_lock_state;
 
 #if	defined(HASPROCFS)
 	case N_PROC:
 	    ni = 1;
 	    if (!v.v_data || kread((KA_T)v.v_data, (char *)&pr, sizeof(pr))) {
-		(void) sprintf(Namech, "vnode@%#x: can't read prnode (%#x)",
-		    na, v.v_data);
+		(void) snpf(Namech, Namechl, "vnode@%s: can't read prnode (%s)",
+		    print_kptr(na, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
@@ -506,50 +555,53 @@ get_lock_state:
 	    i.number = (long)pr.pr_ino;
 	    sd = 0;
 	    if (pr.pr_common
-	    &&  kread((KA_T)pr.pr_common, (char *)&prc, sizeof(prc)) == 0) {
+	    &&  !kread((KA_T)pr.pr_common, (char *)&prc, sizeof(prc))) {
 		pid = (long)prc.prc_pid;
 		switch(pr.pr_type) {
 		case PR_PIDDIR:
-		    (void) sprintf(Namech, "/%s/%ld", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld", HASPROCFS, pid);
 		    break;
 		case PR_AS:
-		    (void) sprintf(Namech, "/%s/%ld/as", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld/as", HASPROCFS, pid);
 		    break;
 		case PR_CTL:
-		    (void) sprintf(Namech, "/%s/%ld/ctl", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld/ctl", HASPROCFS, pid);
 		    break;
 		case PR_STATUS:
-		    (void) sprintf(Namech, "/%s/%ld/status", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld/status", HASPROCFS,
+			pid);
 		    break;
 		case PR_MAP:
-		    (void) sprintf(Namech, "/%s/%ld/map", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld/map", HASPROCFS, pid);
 		    break;
 		case PR_CRED:
-		    (void) sprintf(Namech, "/%s/%ld/cred", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld/cred", HASPROCFS, pid);
 		    break;
 		case PR_SIGACT:
-		    (void) sprintf(Namech, "/%s/%ld/sigact", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld/sigact", HASPROCFS,
+			pid);
 		    break;
 		case PR_OBJECTDIR:
-		    (void) sprintf(Namech, "/%s/%ld/object", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld/object", HASPROCFS,
+			pid);
 		    break;
 		case PR_LWPDIR:
-		    (void) sprintf(Namech, "/%s/%ld/lwp", HASPROCFS, pid);
+		    (void) snpf(Namech, Namechl, "/%s/%ld/lwp", HASPROCFS, pid);
 		    break;
 		case PR_LWPIDDIR:
-		    (void) sprintf(Namech, "/%s/%ld/lwp/%d",
+		    (void) snpf(Namech, Namechl, "/%s/%ld/lwp/%d",
 			HASPROCFS, pid, prc.prc_lwpid);
 		    break;
 		case PR_LWPCTL:
-		    (void) sprintf(Namech, "/%s/%ld/lwp/%d/lwpctl",
+		    (void) snpf(Namech, Namechl, "/%s/%ld/lwp/%d/lwpctl",
 			HASPROCFS, pid, prc.prc_lwpid);
 		    break;
 		case PR_LWPSTATUS:
-		    (void) sprintf(Namech, "/%s/%ld/lwp/%d/lwpstatus",
+		    (void) snpf(Namech, Namechl, "/%s/%ld/lwp/%d/lwpstatus",
 			HASPROCFS, pid, prc.prc_lwpid);
 		    break;
 		case PR_LWPSINFO:
-		    (void) sprintf(Namech, "/%s/%ld/lwp/%d/lwpsinfo",
+		    (void) snpf(Namech, Namechl, "/%s/%ld/lwp/%d/lwpsinfo",
 			HASPROCFS, pid, prc.prc_lwpid);
 		    break;
 		}
@@ -567,21 +619,23 @@ get_lock_state:
 		break;
 	    }
 	    if (kread((KA_T)pr.pr_proc, (char *)&p, sizeof(p))) {
-		(void) sprintf(Namech, "prnode@%#x: can't read proc (%#x)",
-		    v.v_data, pr.pr_proc);
+		(void) snpf(Namech, Namechl, "prnode@%s: can't read proc (%s)",
+		    print_kptr((KA_T)v.v_data, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)pr.pr_proc, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
 	    if (!p.p_pidp || kread((KA_T)p.p_pidp, (char *)&pids, sizeof(pids)))
 	    {
-		(void) sprintf(Namech,
-		    "proc struct at %#x: can't read pid (%#x)",
-		    pr.pr_proc, p.p_pidp);
+		(void) snpf(Namech, Namechl,
+		    "proc struct at %s: can't read pid (%s)",
+		    print_kptr((KA_T)pr.pr_proc, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)p.p_pidp, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
-	    pid = (long)pids.p_pid;
-	    (void) sprintf(Namech, "/%s/%ld", HASPROCFS, pid);
+	    pid = (long)pids.pid_id;
+	    (void) snpf(Namech, Namechl, "/%s/%ld", HASPROCFS, pid);
 	    i.number = pid + PR_INOBIAS;
 	    if (!p.p_as ||  kread((KA_T)p.p_as, (char *)&as, sizeof(as)))
 		sd = 0;
@@ -593,12 +647,12 @@ get_lock_state:
 
 	case N_STREAM:
 	    if (v.v_stream) {
-		Lf->is_stream = 1;
+		Lf->is_stream = ni = 1;
 	    /*
 	     * Get the queue pointer and module name at the end of the stream.
 	     * The module name indentifies socket streams.
 	     */
-		if (examine_stream(v.v_stream, &q, mn, "sockmod", &sqp))
+		if (examine_stream((KA_T)v.v_stream, &q, mn, "sockmod", &sqp))
 		    return;
 		for (px = 0; px < NPROTOS; px++) {
 		    if (strcmp(mn, Protos[px].module) == 0) {
@@ -613,24 +667,20 @@ get_lock_state:
 		if (sqp && kread(sqp, (char *)&so, sizeof(so)) == 0)
 		    break;
 		sqp = (KA_T)NULL;
-		(void) strcpy(Namech, "STR");
+		(void) snpf(Namech, Namechl, "STR");
 		j = strlen(Namech);
 		if (v.v_type == VCHR) {
 	    /*
 	     * If this is a VCHR stream, look up the device name and record it.
-	     *
-	     * Because the stream is also represented by a VCHR device, record
-	     * the device inode number and clear the stream flag so the file
-	     * can be located by the device name.
 	     */
-		    if ((dp = findstrdev((dev_t *)&v.v_rdev))) {
+		    if ((dp = findstrdev(&DevDev, (dev_t *)&v.v_rdev))) {
 			Lf->inode = (unsigned long)dp->inode;
 			Lf->inp_ty = 1;
-			Lf->is_stream = 0;
 			Namech[j++] = ':';
 			k = strlen(dp->name);
-			if ((j + k) <= (MAXPATHLEN - 1)) {
-			    (void) strcpy(&Namech[j], dp->name);
+			if ((j + k) <= (Namechl - 1)) {
+			    (void) snpf(&Namech[j], Namechl - j, "%s",
+				dp->name);
 			    j += k;
 			    if ((cp = strrchr(Namech, '/'))
 			    &&  *(cp + 1) == '\0')
@@ -645,16 +695,16 @@ get_lock_state:
 	     * Follow the "STR" and possibly the device name with "->" and
 	     * the module name.
 	     */
-		if ((j + 2) <= (MAXPATHLEN - 1)) {
-		    (void) strcpy(&Namech[j], "->");
+		if ((j + 2) <= (Namechl - 1)) {
+		    (void) snpf(&Namech[j], Namechl - j, "->");
 		    j += 2;
 		}
 		if (mn[0]) {
-		    if ((j + strlen(mn)) <= (MAXPATHLEN - 1))
-			(void) strcpy(&Namech[j], mn);
+		    if ((j + strlen(mn)) <= (Namechl - 1))
+			(void) snpf(&Namech[j], Namechl - j, mn);
 		} else {
-		    if ((j + strlen("none")) <= (MAXPATHLEN - 1))
-			(void) strcpy(&Namech[j], "none");
+		    if ((j + strlen("none")) <= (Namechl - 1))
+			(void) snpf(&Namech[j], Namechl - j, "none");
 	        }
 	    }
 	    break;
@@ -666,24 +716,33 @@ get_lock_state:
 	 * to its inode.
 	 */
 	    if (v.v_type == VCHR) {
-		if (!v.v_data || readsnode(v.v_data, &s)) {
-		    (void) sprintf(Namech,
-			"vnode@%#x: can't read snode (%#x)", na, v.v_data);
+		if (!v.v_data || readsnode((KA_T)v.v_data, &s)) {
+		    (void) snpf(Namech, Namechl,
+			"vnode@%s: can't read snode (%s)",
+			print_kptr(na, tbuf, sizeof(tbuf)),
+			print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		    enter_nm(Namech);
 		    return;
 		}
 		if (s.s_realvp) {
-		    if (readvnode((caddr_t)s.s_realvp, &rv)) {
-			(void) sprintf(Namech,
-			    "snode@%#x: can't read real vnode (%#x)",
-			    v.v_data, s.s_realvp);
+		    if (readvnode((KA_T)s.s_realvp, &rv)) {
+			(void) snpf(Namech, Namechl,
+			    "snode@%s: can't read real vnode (%s)",
+			    print_kptr((KA_T)v.v_data, tbuf, sizeof(tbuf)),
+			    print_kptr((KA_T)s.s_realvp, (char *)NULL, 0));
 			enter_nm(Namech);
 			return;
 		    }
-		    if (!rv.v_data || readlino(fx, &rv, &i)) {
-			(void) sprintf(Namech,
-			    "snode@%#x: can't read inode (%#x)",
-			     v.v_data, rv.v_data);
+		    if (!rv.v_data || (is = readlino(fx, &rv, &i))) {
+			(void) snpf(Namech, Namechl,
+			    "snode@%s: unknown inode@%s; fx=",
+			    print_kptr((KA_T)v.v_data, tbuf, sizeof(tbuf)),
+			    print_kptr((KA_T)rv.v_data, (char *)NULL, 0));
+			    ep = endnm(&sz);
+			if (fx < 1 || fx > Fsinfomax)
+			    (void) snpf(ep, sz, "%d", fx);
+			else
+			    (void) snpf(ep, sz, "%s", Fsinfo[fx - 1]);
 			enter_nm(Namech);
 			return;
 		     }
@@ -693,38 +752,46 @@ get_lock_state:
 	     * common snode.
 	     */
 		else if (s.s_commonvp) {
-		    if (readvnode((caddr_t)s.s_commonvp, &rv)) {
-			(void) sprintf(Namech,
-			    "snode@%#x: can't read real vnode (%#x)",
-			    v.v_data, s.s_commonvp);
+		    if (readvnode((KA_T)s.s_commonvp, &rv)) {
+			(void) snpf(Namech, Namechl,
+			    "snode@%s: can't read real vnode (%s)",
+			    print_kptr((KA_T)v.v_data, tbuf, sizeof(tbuf)),
+			    print_kptr((KA_T)s.s_commonvp, (char *)NULL, 0));
 			enter_nm(Namech);
 			return;
 		    }
-		    if (!rv.v_data || readsnode(rv.v_data, &rs)) {
-			(void) sprintf(Namech,
-				"vnode@%#x: can't read snode(%#x)",
-				s.s_commonvp, rv.v_data);
+		    if (!rv.v_data || readsnode((KA_T)rv.v_data, &rs)) {
+			(void) snpf(Namech, Namechl,
+			    "vnode@%s: can't read snode(%s)",
+			    print_kptr((KA_T)s.s_commonvp, tbuf, sizeof(tbuf)),
+			    print_kptr((KA_T)rv.v_data, (char *)NULL, 0));
 			enter_nm(Namech);
 			return;
 		    }
-		    ni = 1;
+		    Lf->is_com = ni = 1;
 		} else
 		    ni = 1;
 		break;
 	    }
 	    if (v.v_data == NULL) {
-		(void) sprintf(Namech, "vnode@%#x: no further information",
-		    na);
+		(void) snpf(Namech, Namechl, "vnode@%s: no further information",
+		    print_kptr(na, (char *)NULL, 0));
 		enter_nm(Namech);
 		return;
 	    }
 	/*
 	 * Read inode information.
 	 */
-	    if (readlino(fx, &v, &i)) {
-		(void) sprintf(Namech,
-		    "vnode@%#x: can't read successor node from %#x",
-		    na, v.v_data);
+	    if ((is = readlino(fx, &v, &i))) {
+		(void) snpf(Namech, Namechl,
+		    "vnode@%s: unknown successor@%s; fx=",
+		    print_kptr(na, tbuf, sizeof(tbuf)),
+		    print_kptr((KA_T)v.v_data, (char *)NULL, 0));
+		ep = endnm(&sz);
+		if (fx < 1 || fx > Fsinfomax)
+		    (void) snpf(ep, sz, "%d", fx);
+		else
+		    (void) snpf(ep, sz, "%s", Fsinfo[fx - 1]);
 		enter_nm(Namech);
 		return;
 	    }
@@ -735,25 +802,50 @@ get_lock_state:
 	switch (Ntype) {
 	case N_NFS:
 	    dev = r.r_attr.va_fsid;
+	    devs = 1;
 	    break;
 
 #if	defined(HASPROCFS)
 	case N_PROC:
 	    dev = kv.vfs_dev;
+	    devs = 1;
 	    break;
 #endif	/* defined(HASPROCFS) */
 
 	case N_STREAM:
 	    if (sqp) {
-		dev = so.lux_dev.addr.tu_addr.dev;
+		if (so.lux_dev.size >= 8) {
+		    dev = DevDev;
+		    rdev = so.lux_dev.addr.tu_addr.dev;
+		    devs = rdevs = 1;
+		} else
+		    enter_dev_ch(print_kptr(sqp, (char *)NULL, 0));
+		break;
+	    }
+	    if (v.v_type == VFIFO) {
+		KA_T ta;
+
+		if ((ta = (KA_T)(v.v_data ? v.v_data : v.v_stream)))
+		    enter_dev_ch(print_kptr(ta, (char *)NULL, 0));
 		break;
 	    }
 	    /* fall through */
 	default:
-	    dev = (v.v_type == VCHR || ni) ? v.v_rdev : i.dev;
-	} type = v.v_type;
-	if (lvfs && lvfs->dir == NULL)
-	    (void) completevfs(lvfs, &dev);
+	    if (!ni) {
+		dev = i.dev;
+		devs = 1;
+	    } else if ((Ntype == N_STREAM)
+		   &&  ((v.v_type == VCHR) || (v.v_type == VBLK)))
+	    {
+		dev = DevDev;
+		devs = 1;
+	    }
+	    if ((v.v_type == VCHR) || (v.v_type == VBLK)) {
+		rdev = v.v_rdev;
+		rdevs = 1;
+	    }
+	}
+	type = v.v_type;
 /*
  * Obtain the inode number.
  */
@@ -772,14 +864,16 @@ get_lock_state:
 
 	case N_FIFO:
 	    if (!f.fn_realvp) {
-		(void) sprintf(dev_ch, "0x%08x", v.v_data);
-		enter_dev_ch(dev_ch);
+		enter_dev_ch(print_kptr((KA_T)v.v_data, (char *)NULL, 0));
 		Lf->inode = (unsigned long)f.fn_ino;
 		Lf->inp_ty = 1;
 		if (f.fn_flag & ISPIPE)
-		    (void) strcpy(Namech, "PIPE");
-		if (f.fn_mate)
-		    (void) sprintf(endnm(), "->0x%08x", f.fn_mate);
+		    (void) snpf(Namech, Namechl, "PIPE");
+		if (f.fn_mate) {
+		    ep = endnm(&sz);
+		    (void) snpf(ep, sz, "->%s",
+			print_kptr((KA_T)f.fn_mate, (char *)NULL, 0));
+		}
 		break;
 	    }
 	    /* fall through */
@@ -790,7 +884,7 @@ get_lock_state:
 	    }
 	    break;
 	case N_STREAM:
-	    if (sqp) {
+	    if (sqp && so.lux_dev.size >= 8) {
 		Lf->inode = (unsigned long)so.lux_dev.addr.tu_addr.ino;
 		Lf->inp_ty = 1;
 	    }
@@ -830,40 +924,69 @@ get_lock_state:
 		break;
 	    }
 	}
+/*
+ * Record link count.
+ */
+	if (Fnlink) {
+	    switch(Ntype) {
+	    case N_FIFO:
+		Lf->nlink = (long)f.fn_open;
+		Lf->nlink_def = 1;
+		break;
+	    case N_NFS:
+		Lf->nlink = (long)r.r_attr.va_nlink;
+		Lf->nlink_def = 1;
+		break;
+
+#if	defined(HASPROCFS)
+	    case N_PROC:
+#endif	/* defined(HASPROCFS) */
+
+	    case N_REGLR:
+		if (!ni) {
+		    Lf->nlink = (long)i.nlink;
+		    Lf->nlink_def = i.nlink_def;
+		}
+		break;
+	    }
+	    if (Nlink && Lf->nlink_def && (Lf->nlink < Nlink))
+		Lf->sf |= SELNLINK;
+	}
 /*
  * Record an NFS file selection.
  */
 	if (Ntype == N_NFS && Fnfs)
 	    Lf->sf |= SELNFS;
 /*
- * Save the file system names.
+ * Defer file system info lookup until printname().
  */
-	if (lvfs) {
-	    Lf->fsdir = lvfs->dir;
-	    Lf->fsdev = lvfs->fsname;
-	}
+	Lf->lmi_srch = 1;
 /*
+ * Save the device numbers and their states.
+ *
  * Format the vnode type, and possibly the device name.
  */
+	if (type != VFIFO) {
+	    Lf->dev = dev;
+	    Lf->dev_def = devs;
+	    Lf->rdev = rdev;
+	    Lf->rdev_def = rdevs;
+	}
 	switch (type) {
 	case VNON:
 	    ty ="VNON";
 	    break;
 	case VREG:
+	    ty = "VREG";
+	    break;
 	case VDIR:
-	    ty = (type == VREG) ? "VREG" : "VDIR";
-	    Lf->dev_def = 1;
-	    Lf->dev = dev;
+	    ty = "VDIR";
 	    break;
 	case VBLK:
 	    ty = "VBLK";
-	    Lf->dev = dev;
-	    Lf->dev_def = 1;
 	    Ntype = N_BLK;
 	    break;
 	case VCHR:
-	    Lf->dev = dev;
-	    Lf->dev_def = 1;
 	    ty = "VCHR";
 	    if (Lf->is_stream == 0)
 		Ntype = N_CHR;
@@ -884,7 +1007,9 @@ get_lock_state:
 	case VFIFO:
 	    if (!Lf->dev_ch || Lf->dev_ch[0] == '\0') {
 		Lf->dev = dev;
-		Lf->dev_def = 1;
+		Lf->dev_def = devs;
+		Lf->rdev = rdev;
+		Lf->rdev_def = rdevs;
 	    }
 	    ty = "FIFO";
 	    break;
@@ -893,26 +1018,26 @@ get_lock_state:
 	    break;
 	default:
 	    if (type > 9999)
-		(void) sprintf(Lf->type, "*%03d", type % 1000);
+		(void) snpf(Lf->type, sizeof(Lf->type), "*%03d", type % 1000);
 	    else
-		(void) sprintf(Lf->type, "%4d", type);
-	    (void) strcpy(Namech, "unknown type");
+		(void) snpf(Lf->type, sizeof(Lf->type), "%4d", type);
+	    (void) snpf(Namech, Namechl, "unknown type");
 	    ty = NULL;
 	}
 	if (ty)
-	    (void) strcpy(Lf->type, ty);
+	    (void) snpf(Lf->type, sizeof(Lf->type), "%s", ty);
 	Lf->ntype = Ntype;
 /*
  * If this is a VBLK file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && Lf->ntype == N_BLK && Lf->dev_def)
+	if ((Lf->inp_ty == 0) && (Lf->ntype == N_BLK))
 	    find_bl_ino();
 /*
  * If this is a VCHR file and it's missing an inode number, try to
  * supply one.
  */
-	if (Lf->inp_ty == 0 && type == VCHR && Lf->dev_def)
+	if ((Lf->inp_ty == 0) && (type == VCHR))
 	    find_ch_ino();
 /*
  * If this is a stream with a "sockmod" module whose q_ptr leads to an
@@ -922,12 +1047,23 @@ get_lock_state:
 	if (Ntype == N_STREAM && sqp) {
 	    if (Funix)
 		Lf->sf |= SELUNX;
-	    (void) strcpy(Lf->type, "unix");
+	    (void) snpf(Lf->type, sizeof(Lf->type), "unix");
 	    if (!Namech[0]
 	    &&  so.laddr.buf && so.laddr.len == sizeof(ua)
-	    &&  kread((KA_T)so.laddr.buf, (char *)&ua, sizeof(ua)) == 0) {
+	    &&  !kread((KA_T)so.laddr.buf, (char *)&ua, sizeof(ua))) {
 		ua.sun_path[sizeof(ua.sun_path) - 1] = '\0';
-		(void) strcpy(Namech, ua.sun_path);
+		(void) snpf(Namech, Namechl, "%s", ua.sun_path);
+		if (Sfile && is_file_named(Namech, 0))
+		    Lf->sf = SELNM;
+		if (so.lux_dev.size >= 8) {
+		    Lf->inode = (unsigned long)so.lux_dev.addr.tu_addr.ino;
+		    Lf->inp_ty = 1;
+		}
+	    }
+	    if (so.so_conn) {
+		ep = endnm(&sz);
+		(void) snpf(ep, sz, "->%s",
+		    print_kptr((KA_T)so.so_conn, (char *)NULL, 0));
 	    }
 	    Lf->is_stream = 0;
 	}
@@ -953,8 +1089,8 @@ get_lock_state:
 			pfi->f = 1;
 			Lf->sf |= SELNM;
 			if (!Namech[0] && pfi->nm) {
-			    (void) strncpy(Namech, pfi->nm, MAXPATHLEN - 1);
-			    Namech[MAXPATHLEN-1] = '\0';
+			    (void) strncpy(Namech, pfi->nm, Namechl - 1);
+			    Namech[Namechl-1] = '\0';
 			}
 			break;
 		    }
@@ -964,12 +1100,28 @@ get_lock_state:
 #endif	/* defined(HASPROCFS) */
 
 	{
-	    if (Sfile && is_file_named(NULL, type))
+	    if (Sfile && is_file_named((char *)NULL,
+				       ((type == VCHR) || (type == VBLK)) ? 1
+									  : 0))
 		Lf->sf |= SELNM;
 	}
 /*
- * Enter name characters.
+ * Enter name characters.  If there's an l_ino structure with a file name
+ * pointer, and no name column addition exists, make what the l_ino file
+ * name pointer addresses a name column addition.
  */
+	if (!Lf->nma && !is && i.nm) {
+	    if ((msz = (MALLOC_S)strlen(i.nm))) {
+		if (!(cp = (char *)malloc(msz + 1))) {
+		    (void) fprintf(stderr,
+			"%s: can't allocate %d bytes for l_ino name addition\n",
+			msz, Pn);
+		    Exit(1);
+		}
+		(void) snpf(cp, msz + 1, "%s", i.nm);
+		Lf->nma = cp;
+	    }
+	}
 	if (Namech[0])
 	    enter_nm(Namech);
 }
@@ -986,43 +1138,71 @@ readlino(fx, v, i)
 	struct l_ino *i;		/* local inode */
 {
 	struct vnode fa;
-	struct vfs kv;
 	struct mnode mn;
 	struct inode sn;
-	struct xnamnode xn;
 
-	if (fx < 1 || fx > Fsinfomax || v->v_data == NULL)
+#if	defined(HASXNAMNODE)
+	struct xnamnode xn;
+#endif	/* defined(HASXNAMNODE) */
+
+	i->nlink_def = 0;
+	if (fx < 1 || fx > Fsinfomax || !v->v_data)
 	    return(1);
-	if (strcmp(Fsinfo[fx-1], "fifofs") == 0
-	||  strcmp(Fsinfo[fx-1], "ufs") == 0) {
+	if (!strcmp(Fsinfo[fx-1], "fifofs")
+	||  !strcmp(Fsinfo[fx-1], "sfs")
+	||  !strcmp(Fsinfo[fx-1], "ufs")) {
 	    if (kread((KA_T)v->v_data, (char *)&sn, sizeof(sn)))
 		return(1);
 	    i->dev = sn.i_dev;
 	    i->rdev = v->v_rdev;
+	    i->nlink = (long)sn.i_nlink;
+	    i->nlink_def = 1;
+	    i->nm = (char *)NULL;
 	    i->number = sn.i_number;
 	    i->size = sn.i_size;
 	    return(0);
-	} else if (strcmp(Fsinfo[fx-1], "s5") == 0)
+	} else if (!strcmp(Fsinfo[fx-1], "s5"))
 	    return(reads5lino(v, i));
- 	else if (strcmp(Fsinfo[fx-1], "vxfs") == 0)
+ 	else if (!strcmp(Fsinfo[fx-1], "vxfs"))
 	    return(readvxfslino(v, i));
-	else if (strcmp(Fsinfo[fx-1], "xnamfs") == 0
-	     ||  strcmp(Fsinfo[fx-1], "XENIX") == 0)
+ 	else if (!strcmp(Fsinfo[fx-1], "bfs"))
+	    return(readbfslino(v, i));
+
+#if	defined(HASXNAMNODE)
+	else if (!strcmp(Fsinfo[fx-1], "xnamfs")
+	     ||  !strcmp(Fsinfo[fx-1], "XENIX"))
 	{
 	    if (kread((KA_T)v->v_data, (char *)&xn, sizeof(xn)))
 		return(1);
 	    i->dev = xn.x_dev;
+	    i->nlink = (long)xn.x_count;
+	    i->nlink_def = 1;
+	    i->nm = (char *)NULL;
 	    i->rdev = xn.x_fsid;
 	    i->size = xn.x_size;
 	    return(0);
-	} else if (strcmp(Fsinfo[fx-1], "memfs") == 0) {
+	}
+#endif	/* defined(HASXNAMNODE) */
+
+	else if (!strcmp(Fsinfo[fx-1], "memfs")) {
 	    if (kread((KA_T)v->v_data, (char *)&mn, sizeof(mn)))
 		return(1);
 	    i->dev = mn.mno_fsid;
+	    i->nlink = (long)mn.mno_nlink;
+	    i->nlink_def = 1;
+	    i->nm = (char *)NULL;
 	    i->number = mn.mno_nodeid;
 	    i->rdev = mn.mno_rdev;
 	    i->size = mn.mno_size;
 	    return(0);
 	}
+
+#if	UNIXWAREV>=70000
+	else if (!strcmp(Fsinfo[fx-1], "cdfs"))
+	    return readcdfslino(v, i);
+	else if (!strcmp(Fsinfo[fx-1], "dosfs"))
+	    return readdosfslino(v, i);
+#endif	/* UNIXWAREV>=70000 */
+
 	return(1);
 }
