@@ -34,7 +34,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: main.c,v 1.16 98/03/06 07:55:00 abe Exp $";
+static char *rcsid = "$Id: main.c,v 1.32 2000/12/05 20:32:46 abe Exp $";
 #endif
 
 
@@ -65,15 +65,18 @@ main(argc, argv)
 	int argc;
 	char *argv[];
 {
-	int c, ev, i, n, rv;
+	int c, i, n, rv;
 	char *cp;
 	int err = 0;
+	int ev = 0;
 	int fh = 0;
+	long l;
 	MALLOC_S len;
 	struct lfile *lf;
 	struct nwad *np, *npn;
 	char options[128];
 	int rc = 0;
+	struct stat sb;
 	struct sfile *sfp;
 	struct lproc **slp = (struct lproc **)NULL;
 	int sp = 0;
@@ -96,11 +99,12 @@ main(argc, argv)
 	    (void) fprintf(stderr, "%s: no space for name buffer\n", Pn);
 	    Exit(1);
 	}
+	Namechl = (size_t)(MAXPATHLEN + 1);
 /*
  * Create option mask.
  */
-	(void) sprintf(options,
-	    "?a%sbc:%sd:%sfF:g:hi:%sl%sMnNo:Op:Pr:%ssS:tT:u:UvVw%s",
+	(void) snpf(options, sizeof(options),
+	    "?a%sbc:D:d:%sf:F:g:hi:%slL:%sMnNo:Op:Pr:%ssS:tT:u:UvVw%s",
 
 #if	defined(HAS_AFS) && defined(HASAOPT)
 	    "A:",
@@ -114,12 +118,6 @@ main(argc, argv)
 	    "",
 #endif	/* defined(HASNCACHE) */
 
-#if	defined(HASDCACHE)
-	    "D:",
-#else	/* !defined(HASDCACHE) */
-	    "",
-#endif	/* defined(HASDCACHE) */
-
 #if	defined(HASKOPT)
 	    "k:",
 #else	/* !defined(HASKOPT) */
@@ -187,19 +185,84 @@ main(argc, argv)
 #endif	/* defined(HASNCACHE) */
 
 	    case 'd':
-		if (enter_fd(GOv))
-		    err = 1;
+		if (GOp == '+') {
+		    if (enter_dir(GOv, 0))
+			err = 1;
+		    else
+			Selflags |= SELNM;
+		} else {
+		    if (enter_fd(GOv))
+			err = 1;
+		}
 		break;
+	    case 'D':
+		if (GOp == '+') {
+		    if (enter_dir(GOv, 1))
+			err = 1;
+		    else
+			Selflags |= SELNM;
+		} else {
 
 #if	defined(HASDCACHE)
-	    case 'D':
-		if (ctrl_dcache(GOv))
+		    if (ctrl_dcache(GOv))
+			err = 1;
+#else	/* !defined(HASDCACHE) */
+		    (void) fprintf(stderr, "%s: unsupported option: -D\n", Pn);
 		    err = 1;
-		break;
 #endif	/* defined(HASDCACHE) */
 
+		}
+		break;
 	    case 'f':
-		Ffilesys = (GOp == '+') ? 2 : 1;
+		if (!GOv || *GOv == '-' || *GOv == '+') {
+		    Ffilesys = (GOp == '+') ? 2 : 1;
+		    break;
+		}
+
+#if	defined(HASFSTRUCT)
+		for (; *GOv; GOv++) {
+		    switch (*GOv) {
+		    case 'c':
+		    case 'C':
+			if (GOp == '+')
+			    Fsv |= FSV_CT;
+			else
+			    Fsv &= (unsigned char)~FSV_CT;
+			break;
+		    case 'f':
+		    case 'F':
+			if (GOp == '+')
+			    Fsv |= FSV_FA;
+			else
+			    Fsv &= (unsigned char)~FSV_FA;
+			break;
+		    case 'g':
+		    case 'G':
+			if (GOp == '+')
+			    Fsv |= FSV_FG;
+			else
+			    Fsv &= (unsigned char)~FSV_FG;
+			FsvFlagX = (*GOv == 'G') ? 1 : 0;
+			break;
+		    case 'n':
+		    case 'N':
+			if (GOp == '+')
+			    Fsv |= FSV_NI;
+			else
+			    Fsv &= (unsigned char)~FSV_NI;
+			break;
+		    default:
+			(void) fprintf(stderr,
+			    "%s: unknown file struct option: %c\n", Pn, *GOv);
+			err++;
+		    }
+		}
+#else	/* !defined(HASFSTRUCT) */
+		(void) fprintf(stderr,
+		    "%s: unknown string for %cf: %s\n", Pn, GOp, GOv);
+		err++;
+#endif	/* defined(HASFSTRUCT) */
+
 		break;
 	    case 'F':
 		if (!GOv || *GOv == '-' || *GOv == '+'
@@ -218,9 +281,25 @@ main(argc, argv)
 			    continue;
 #endif	/* !defined(HASPPID) */
 
+#if	!defined(HASFSTRUCT)
+			if (FieldSel[i].id == LSOF_FID_CT
+			||  FieldSel[i].id == LSOF_FID_FA
+			||  FieldSel[i].id == LSOF_FID_FG
+			||  FieldSel[i].id == LSOF_FID_NI)
+			    continue;
+#endif	/* !defined(HASFSTRUCT) */
+
 			FieldSel[i].st = 1;
+			if (FieldSel[i].opt && FieldSel[i].ov)
+			    *(FieldSel[i].opt) |= FieldSel[i].ov;
 		    }
+
+#if	defined(HASFSTRUCT)
+		    Ffield = FsvFlagX = 1;
+#else	/* !defined(HASFSTRUCT) */
 		    Ffield = 1;
+#endif	/* defined(HASFSTRUCT) */
+
 		    break;
 		}
 		if (strcmp(GOv, "?") == 0) {
@@ -235,8 +314,24 @@ main(argc, argv)
 			    continue;
 #endif	/* !defined(HASPPID) */
 
+#if	!defined(HASFSTRUCT)
+			if (FieldSel[i].id == LSOF_FID_CT
+			||  FieldSel[i].id == LSOF_FID_FA
+			||  FieldSel[i].id == LSOF_FID_FG
+			||  FieldSel[i].id == LSOF_FID_NI)
+			    continue;
+#endif	/* !defined(HASFSTRUCT) */
+
 			if (FieldSel[i].id == *GOv) {
 			    FieldSel[i].st = 1;
+			    if (FieldSel[i].opt && FieldSel[i].ov)
+				*(FieldSel[i].opt) |= FieldSel[i].ov;
+
+#if	defined(HASFSTRUCT)
+			    if (i == LSOF_FIX_FG)
+				FsvFlagX = 1;
+#endif	/* defined(HASFSTRUCT) */
+
 			    if (i == LSOF_FIX_TERM)
 				Terminator = '\0';
 			    break;
@@ -290,6 +385,38 @@ main(argc, argv)
 	    case 'l':
 		Futol = 0;
 		break;
+	    case 'L':
+		Fnlink = (GOp == '+') ? 1 : 0;
+		if (!GOv || *GOv == '-' || *GOv == '+') {
+		    if (GOv) {
+			GOx1 = GObk[0];
+			GOx2 = GObk[1];
+		    }
+		    Nlink = 0l;
+		    break;
+		}
+		for (cp = GOv, l = 0l, n = 0; *cp; cp++) {
+		    if (!isdigit((unsigned char)*cp))
+			break;
+		    l = (l * 10l) + ((long)*cp - (long)'0');
+		    n++;
+		}
+		if (n) {
+		    if (GOp != '+') {
+			(void) fprintf(stderr,
+			    "%s: no number may follow -L\n", Pn);
+			err = 1;
+		    } else {
+			Nlink = l;
+			Selflags |= SELNLINK;
+		    }
+		} else
+		    Nlink = 0l;
+		if (*cp) {
+		    GOx1 = GObk[0];
+		    GOx2 = GObk[1] + n;
+		}
+		break;
 
 #if	defined(HASMOPT)
 	    case 'm':
@@ -510,7 +637,7 @@ main(argc, argv)
 	if (Cmdl)
 	    Selflags |= SELCMD;
 	if (Fdl)
-		Selflags |= SELFD;
+	    Selflags |= SELFD;
 	if (Fnet)
 	    Selflags |= SELNET;
 	if (Fnfs)
@@ -541,21 +668,27 @@ main(argc, argv)
 	    Selall = 0;
 	}
 /*
- * Read the mount table, unless listing only network files (-i option).
+ * Get the device for /dev.
  */
-	if (Selinet == 0 && readmnt() == 0)
+	if (stat("/dev", &sb)) {
+	    (void) fprintf(stderr, "%s: can't stat(/dev): %s\n", Pn,
+		strerror(errno));
 	    Exit(1);
+	}
+	DevDev = sb.st_dev;
 /*
  * Process the file arguments.
  */
 	if (GOx1 < argc) {
-	    if (ck_file_arg(GOx1, argc, argv))
+	    if (ck_file_arg(GOx1, argc, argv, Ffilesys, 0, (struct stat *)NULL))
 		usage(1, 0, 0);
 	}
 /*
  * Do dialect-specific initialization.
  */
 	initialize();
+	if (Sfile)
+	    (void) hashSfile();
 
 #if	defined(WILLDROPGID)
 /*
@@ -565,6 +698,7 @@ main(argc, argv)
 	(void) dropgid();
 #endif	/* defined(WILLDROPGID) */
 
+
 #if	defined(HASDCACHE)
 /*
  * If there is a device cache, prepare the device table.
@@ -576,13 +710,13 @@ main(argc, argv)
 /*
  * Define the size and offset print formats.
  */
-	(void) sprintf(options, "0t%%%su", SZOFFPSPEC);
+	(void) snpf(options, sizeof(options), "0t%%%su", SZOFFPSPEC);
 	SzOffFmt_0t = sv_fmt_str(options);
-	(void) sprintf(options, "%%%su", SZOFFPSPEC);
+	(void) snpf(options, sizeof(options), "%%%su", SZOFFPSPEC);
 	SzOffFmt_d = sv_fmt_str(options);
-	(void) sprintf(options, "%%*%su", SZOFFPSPEC);
+	(void) snpf(options, sizeof(options), "%%*%su", SZOFFPSPEC);
 	SzOffFmt_dv = sv_fmt_str(options);
-	(void) sprintf(options, "%%#%sx", SZOFFPSPEC);
+	(void) snpf(options, sizeof(options), "%%#%sx", SZOFFPSPEC);
 	SzOffFmt_x = sv_fmt_str(options);
 /*
  * Gather and report process information every RptTm seconds.
@@ -620,6 +754,13 @@ main(argc, argv)
 	    }
 	    if ((n = Nlproc)) {
 
+#if	defined(HASNCACHE)
+	    /*
+	     * If using the kernel name cache, force its reloading.
+	     */
+		NcacheReload = 1;
+#endif	/* defined(HASNCACHE) */
+
 	    /*
 	     * Print the selected processes and count them.
 	     *
@@ -631,8 +772,8 @@ main(argc, argv)
 		    for (i = n = 0; i < Nlproc; i++) {
 			Lp = (Nlproc > 1) ? slp[i] : &Lproc[i];
 			if (Lp->pss) {
-			    print_proc();
-			    n++;
+			    if (print_proc())
+				n++;
 			}
 			if (RptTm && PrPass)
 			    (void) free_lproc(Lp);
@@ -724,24 +865,46 @@ main(argc, argv)
 	    }
 #endif	/* defined(HASPROCFS) */
 
-	for (np = Nwad; np; np = npn) {
+	if ((np = Nwad)) {
 
 	/*
 	 * Check Internet address specifications.
 	 *
-	 * If any Internet address derived from the same argument was located,
-	 * consider all derivations located.
+	 * If any Internet address derived from the same argument was found,
+	 * consider all derivations found.  If no derivation from the same
+	 * argument was found, report only the first failure.
+	 *
 	 */
-	    for (cp = np->arg, i = 1;; np = npn) {
-		if (np->f)
-		    i = 0;
-		if (!(npn = np->next) || cp != npn->arg)
-		    break;
+	    for (; np; np = np->next) {
+		if (!(cp = np->arg))
+		    continue;
+		for (npn = np->next; npn; npn = npn->next) {
+		    if (!npn->arg)
+			continue;
+		    if (!strcmp(cp, npn->arg)) {
+
+		    /*
+		     * If either of the duplicate specifications was found,
+		     * mark them both found.  If neither was found, mark all
+		     * but the first one found.
+		     */
+			if (np->f)
+			    npn->f = np->f;
+			else if (npn->f)
+			    np->f = npn->f;
+			else
+			    npn->f = 1;
+		    }
+		}
 	    }
-	    rv |= i;
-	    if (i && Fverbose) {
-		(void) printf("%s: Internet address not located: ", Pn);
-		safestrprt(cp ? cp : "(unknown)", stdout, 1);
+	    for (np = Nwad; np; np = np->next) {
+		if (!np->f && (cp = np->arg)) {
+		    rv = 1;
+		    if (Fverbose) {
+			(void) printf("%s: Internet address not located: ", Pn);
+			safestrprt(cp ? cp : "(unknown)", stdout, 1);
+		    }
+		}
 	    }
 	}
 	if (Fnet && Fnet < 2) {
@@ -806,6 +969,8 @@ main(argc, argv)
 	}
 	if (!rv && rc)
 	    rv = ev;
+	if (!rv && ErrStat)
+	    rv = 1;
 	Exit(rv);
 }
 
@@ -955,6 +1120,6 @@ sv_fmt_str(f)
 		"%s: can't allocate %d bytes for format: %s\n", Pn, (int)l, f);
 	    Exit(1);
 	}
-	(void) strcpy(cp, f);
+	(void) snpf(cp, l, "%s", f);
 	return(cp);
 }
