@@ -32,13 +32,50 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: proc.c,v 1.25 2000/03/16 15:25:53 abe Exp $";
+static char *rcsid = "$Id: proc.c,v 1.27 2000/12/04 13:54:42 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
 
+/*
+ * add_nma() - add to NAME column addition
+ */
+
+void
+add_nma(cp, len)
+	char *cp;			/* string to add */
+	int len;			/* string length */
+{
+	int nl;
+
+	if (!cp || !len)
+	    return;
+	if (Lf->nma) {
+	    nl = (int)strlen(Lf->nma);
+	    Lf->nma = (char *)realloc((MALLOC_P *)Lf->nma,
+				      (MALLOC_S)(len + nl + 2));
+	} else {
+	    nl = 0;
+	    Lf->nma = (char *)malloc((MALLOC_S)(len + 1));
+	}
+	if (!Lf->nma) {
+	    (void) fprintf(stderr, "%s: no name addition space: PID %ld, FD %s",
+		Pn, (long)Lp->pid, Lf->fd);
+	    Exit(1);
+	}
+	if (nl) {
+	    Lf->nma[nl] = ' ';
+	    (void) strncpy(&Lf->nma[nl + 1], cp, len);
+	    Lf->nma[nl + 1 + len] = '\0';
+	} else {
+	    (void) strncpy(Lf->nma, cp, len);
+	    Lf->nma[len] = '\0';
+	}
+}
+
+
 #if	defined(HASFSTRUCT)
 _PROTOTYPE(static char *alloc_fflbuf,(char **bp, int *al, int lr));
 
@@ -110,6 +147,7 @@ alloc_lfile(nm, num)
 	Lf->access = Lf->lock = ' ';
 	Lf->dev_def = Lf->inp_ty = Lf->is_com = Lf->is_nfs = Lf->is_stream
 		    = Lf->lmi_srch = Lf->nlink_def = Lf->off_def = Lf->sz_def
+		    = Lf->rdev_def
 		    = (unsigned char)0;
 	Lf->li[0].af = Lf->li[1].af = 0;
 	Lf->lts.type = -1;
@@ -142,9 +180,9 @@ alloc_lfile(nm, num)
 	    Lf->fd[FDLEN - 1] = '\0';
 	} else if (num >= 0) {
 	    if (num < 10000)
-		(void) sprintf(Lf->fd, "%4d", num);
+		(void) snpf(Lf->fd, sizeof(Lf->fd), "%4d", num);
 	    else
-		(void) sprintf(Lf->fd, "*%03d", num % 1000);
+		(void) snpf(Lf->fd, sizeof(Lf->fd), "*%03d", num % 1000);
 	} else
 	    Lf->fd[0] = '\0';
 	Lf->dev_ch = Lf->fsdir = Lf->fsdev = Lf->nm = Lf->nma = (char *)NULL;
@@ -643,8 +681,8 @@ print_fflags(ffg, pof)
 	long pof;		/* process open files flags value */
 {
 	int al, ct, fx;
-	static char *bp = (char *)NULL;
 	static int bl = 0;
+	static char *bp = (char *)NULL;
 	char *sep;
 	int sepl;
 	struct pff_tab *tp;
@@ -676,7 +714,7 @@ print_fflags(ffg, pof)
 		    break;
 		al = (int)strlen(tp->nm) + sepl;
 		bp = alloc_fflbuf(&bp, &bl, al + ct);
-		(void) sprintf((bp + ct), "%s%s", sep, tp->nm);
+		(void) snpf(bp + ct, al + 1, "%s%s", sep, tp->nm);
 		sep = ",";
 		sepl = 1;
 		wf &= ~(tp->val);
@@ -687,10 +725,10 @@ print_fflags(ffg, pof)
 	 * in hex.
 	 */
 	    if (wf || FsvFlagX) {
-		(void) sprintf(xbuf, "0x%lx", wf);
+		(void) snpf(xbuf, sizeof(xbuf), "0x%lx", wf);
 		al = (int)strlen(xbuf) + sepl;
 		bp = alloc_fflbuf(&bp, &bl, al + ct);
-		(void) sprintf((bp + ct), "%s%s", sep, xbuf);
+		(void) snpf(bp + ct, al + 1, "%s%s", sep, xbuf);
 		ct += al;
 	    }
 	}
@@ -826,7 +864,7 @@ print_proc()
 	    if (FieldSel[LSOF_FIX_FA].st && (Fsv & FSV_FA)
 	    &&  (Lf->fsv & FSV_FA)) {
 		(void) printf("%c%s%c", LSOF_FID_FA,
-		    print_kptr(Lf->fsa, (char *)NULL), Terminator);
+		    print_kptr(Lf->fsa, (char *)NULL, 0), Terminator);
 		lc++;
 	    }
 	    if (FieldSel[LSOF_FIX_CT].st && (Fsv & FSV_CT)
@@ -843,7 +881,7 @@ print_proc()
 	    if (FieldSel[LSOF_FIX_NI].st && (Fsv & FSV_NI)
 	    &&  (Lf->fsv & FSV_NI)) {
 		(void) printf("%c%s%c", LSOF_FID_NI,
-		    print_kptr(Lf->fna, (char *)NULL), Terminator);
+		    print_kptr(Lf->fna, (char *)NULL, 0), Terminator);
 		lc++;
 	    }
 #endif	/* defined(HASFSTRUCT) */
