@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1998 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: usage.c,v 1.12 2000/05/31 12:59:06 abe Exp $";
+static char *rcsid = "$Id: usage.c,v 1.13 2000/08/01 17:18:24 abe Exp $";
 #endif
 
 
@@ -185,13 +185,13 @@ usage(xv, fh, version)
 
 	    else
 		cp = "?|i|b|r|u[path]";
-	    (void) sprintf(buf, "-D D  %s", cp);
+	    (void) snpf(buf, sizeof(buf), "-D D  %s", cp);
 #else	/* !defined(HASDCACHE) */
-	    (void) strcpy(buf, " ");
+	    (void) snpf(buf, sizeof(buf), " ");
 #endif	/* defined(HASDCACHE) */
 
 	    (void) fprintf(stderr, "  %s\n", buf);
-	    (void) sprintf(buf, "-i select IPv%s files",
+	    (void) snpf(buf, sizeof(buf), "-i select IPv%s files",
 
 #if	defined(HASIPv6)
 			  "[46]"
@@ -222,7 +222,7 @@ usage(xv, fh, version)
 	    (void) fprintf(stderr, "  %s\n", "-U select Unix socket");
 	    (void) fprintf(stderr, "  %-23.23s", "-v list version info");
 	    (void) fprintf(stderr, "  %-25.25s", "-V verbose search");
-	    (void) sprintf(buf, "+|-w  Warnings (%s)",
+	    (void) snpf(buf, sizeof(buf), "+|-w  Warnings (%s)",
 
 #if	defined(WARNINGSTATE)
 		"-"
@@ -236,9 +236,9 @@ usage(xv, fh, version)
 #if	defined(HASXOPT)
 # if	defined(HASXOPT_ROOT)
 	    if (Myuid == 0)
-		(void) sprintf(buf, "-X %s", HASXOPT);
+		(void) snpf(buf, sizeof(buf), "-X %s", HASXOPT);
 # else	/* !defined(HASXOPT_ROOT) */
-	    (void) sprintf(buf, "-X %s", HASXOPT);
+	    (void) snpf(buf, sizeof(buf), "-X %s", HASXOPT);
 # endif	/* defined(HASXOPT_ROOT) */
 	    (void) fprintf(stderr, "  %-23.23s", buf);
 #endif	/* defined(HASXOPT) */
@@ -284,13 +284,13 @@ usage(xv, fh, version)
 		"  +|-L [l] list (+) suppress (-) link counts < l (0 = all; default = 0)\n");
 
 # if	defined(HASMOPT)
-	    (void) sprintf(buf, "-m m   kernel memory (%s)", KMEM);
+	    (void) snpf(buf, sizeof(buf), "-m m   kernel memory (%s)", KMEM);
 #else	/* !defined(HASMOPT) */
-	    (void) strcpy(buf, " ");
+	    (void) snpf(buf, sizeof(buf), " ");
 #endif	/* defined(HASMOPT) */
 
 	    (void) fprintf(stderr, "  %-36.36s", buf);
-	    (void) sprintf(buf, "+|-M   portMap registration (%s)",
+	    (void) snpf(buf, sizeof(buf), "+|-M   portMap registration (%s)",
 
 #if	defined(HASPMAPENABLED)
 		"+"
@@ -300,14 +300,14 @@ usage(xv, fh, version)
 
 	    );
 	    (void) fprintf(stderr, "  %s\n", buf);
-	    (void) sprintf(buf, "-o o   o 0t offset digits (%d)",
+	    (void) snpf(buf, sizeof(buf), "-o o   o 0t offset digits (%d)",
 		OFFDECDIG);
 	    (void) fprintf(stderr, "  %-36.36s", buf);
 	    (void) fprintf(stderr, "  -p s   select by PID set\n");
-	    (void) sprintf(buf,
+	    (void) snpf(buf, sizeof(buf),
 		"-S [t] t second stat timeout (%d)", TMLIMIT);
 	    (void) fprintf(stderr, "  %-36.36s", buf);
-	    (void) sprintf(buf, "-T %ss%s TCP/TPI %sSt%s info (s)",
+	    (void) snpf(buf, sizeof(buf), "-T %ss%s TCP/TPI %sSt%s info (s)",
 
 #if 	defined(HASTCPTPIQ)
 		"q",
